# Copyright (c) 2011 - 2022 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

import tempfile
import unittest
from unittest import mock
from unittest.mock import patch

from rhsm.profile import ModulesProfile, EnabledReposProfile


class TestModulesProfile(unittest.TestCase):
    """
    Class for testing ModulesProfile (reporting of modulemd in combined profile)
    """

    def setUp(self) -> None:
        # Mock dnf module
        dnf_patcher = patch("rhsm.profile.dnf")
        self.dnf_mock = dnf_patcher.start()
        self.addCleanup(dnf_patcher.stop)
        # Mock libdnf module
        libdnf_patcher = patch("rhsm.profile.libdnf")
        self.libdnf_mock = libdnf_patcher.start()
        self.addCleanup(libdnf_patcher.stop)
        # Mock cloud provider
        cloud_provider_patcher = patch("rhsm.profile.provider")
        self.cloud_provider_mock = cloud_provider_patcher.start()
        self.cloud_provider_mock.get_cloud_provider = mock.Mock(return_value=None)
        self.addCleanup(cloud_provider_patcher.stop)

    def test_default_status(self) -> None:
        """
        Test the case, when module is not enabled nor disabled. The status of module
        should be default
        """
        base_mock = mock.Mock()
        base_mock._moduleContainer = mock.Mock()
        base_mock._moduleContainer.isEnabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isDisabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isModuleActive = mock.Mock(return_value=False)
        module_pkg_mock = mock.Mock()
        profile_mock = mock.Mock()
        profile_mock.getName = mock.Mock(return_value="mock_profile")
        module_pkg_mock.getProfiles = mock.Mock(return_value=[profile_mock])
        module_pkg_mock.getName = mock.Mock(return_value="mock_module")
        module_pkg_mock.getStream = mock.Mock(return_value="1.23")
        module_pkg_mock.getVersion = mock.Mock(return_value="123456789")
        module_pkg_mock.getContext = mock.Mock(return_value="abcdefgh")
        module_pkg_mock.getArch = mock.Mock(return_value="x86_64")
        self.all_module_list = [module_pkg_mock]
        base_mock._moduleContainer.getModulePackages = mock.Mock(return_value=self.all_module_list)
        self.dnf_mock.Base = mock.Mock(return_value=base_mock)

        modules_profile = ModulesProfile()

        self.assertEqual(len(modules_profile.content), 1)
        self.assertEqual(modules_profile.content[0]["status"], "default")
        self.assertEqual(modules_profile.content[0]["active"], False)

    def test_default_status_active(self) -> None:
        """
        Test the case, when module is not enabled nor disabled. The status of module
        should be default. Test the case, when the module can be active despite it
        is in default state.
        """
        base_mock = mock.Mock()
        base_mock._moduleContainer = mock.Mock()
        base_mock._moduleContainer.isEnabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isDisabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isModuleActive = mock.Mock(return_value=True)
        module_pkg_mock = mock.Mock()
        profile_mock = mock.Mock()
        profile_mock.getName = mock.Mock(return_value="mock_profile")
        module_pkg_mock.getProfiles = mock.Mock(return_value=[profile_mock])
        module_pkg_mock.getName = mock.Mock(return_value="mock_module")
        module_pkg_mock.getStream = mock.Mock(return_value="1.23")
        module_pkg_mock.getVersion = mock.Mock(return_value="123456789")
        module_pkg_mock.getContext = mock.Mock(return_value="abcdefgh")
        module_pkg_mock.getArch = mock.Mock(return_value="x86_64")
        self.all_module_list = [module_pkg_mock]
        base_mock._moduleContainer.getModulePackages = mock.Mock(return_value=self.all_module_list)
        self.dnf_mock.Base = mock.Mock(return_value=base_mock)

        modules_profile = ModulesProfile()

        self.assertEqual(len(modules_profile.content), 1)
        self.assertEqual(modules_profile.content[0]["status"], "default")
        self.assertEqual(modules_profile.content[0]["active"], True)

    def test_disabled_status(self) -> None:
        """
        Test the case, when module is disabled.
        """
        base_mock = mock.Mock()
        base_mock._moduleContainer = mock.Mock()
        base_mock._moduleContainer.isEnabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isDisabled = mock.Mock(return_value=True)
        base_mock._moduleContainer.isModuleActive = mock.Mock(return_value=False)
        module_pkg_mock = mock.Mock()
        profile_mock = mock.Mock()
        profile_mock.getName = mock.Mock(return_value="mock_profile")
        module_pkg_mock.getProfiles = mock.Mock(return_value=[profile_mock])
        module_pkg_mock.getName = mock.Mock(return_value="mock_module")
        module_pkg_mock.getStream = mock.Mock(return_value="1.23")
        module_pkg_mock.getVersion = mock.Mock(return_value="123456789")
        module_pkg_mock.getContext = mock.Mock(return_value="abcdefgh")
        module_pkg_mock.getArch = mock.Mock(return_value="x86_64")
        self.all_module_list = [module_pkg_mock]
        base_mock._moduleContainer.getModulePackages = mock.Mock(return_value=self.all_module_list)
        self.dnf_mock.Base = mock.Mock(return_value=base_mock)

        modules_profile = ModulesProfile()

        self.assertEqual(len(modules_profile.content), 1)
        self.assertEqual(modules_profile.content[0]["status"], "disabled")
        self.assertEqual(modules_profile.content[0]["active"], False)

    def test_enabled_status(self) -> None:
        """
        Test the case, when module is enabled.
        """
        base_mock = mock.Mock()
        base_mock._moduleContainer = mock.Mock()
        base_mock._moduleContainer.isEnabled = mock.Mock(return_value=True)
        base_mock._moduleContainer.isDisabled = mock.Mock(return_value=False)
        base_mock._moduleContainer.isModuleActive = mock.Mock(return_value=True)
        module_pkg_mock = mock.Mock()
        profile_mock1 = mock.Mock()
        profile_mock1.getName = mock.Mock(return_value="mock_profile1")
        profile_mock2 = mock.Mock()
        profile_mock2.getName = mock.Mock(return_value="mock_profile2")
        module_pkg_mock.getProfiles = mock.Mock(return_value=[profile_mock1, profile_mock2])
        base_mock._moduleContainer.getInstalledProfiles = mock.Mock(return_value=("mock_profile1",))
        module_pkg_mock.getName = mock.Mock(return_value="mock_module")
        module_pkg_mock.getStream = mock.Mock(return_value="1.23")
        module_pkg_mock.getVersion = mock.Mock(return_value="123456789")
        module_pkg_mock.getContext = mock.Mock(return_value="abcdefgh")
        module_pkg_mock.getArch = mock.Mock(return_value="x86_64")
        self.all_module_list = [module_pkg_mock]
        base_mock._moduleContainer.getModulePackages = mock.Mock(return_value=self.all_module_list)
        self.dnf_mock.Base = mock.Mock(return_value=base_mock)

        modules_profile = ModulesProfile()

        self.assertEqual(len(modules_profile.content), 1)
        self.assertEqual(modules_profile.content[0]["status"], "enabled")
        self.assertEqual(modules_profile.content[0]["active"], True)


REPO_FILE_CONTENT = """
[slick-catlike-tools-1-rpms]
name = Slick Catlike Tools
baseurl = http://cdn.foo.com/content/dist/cats/1.0/$basearch/os
enabled = 0
gpgcheck = 1
gpgkey = file://
sslverify = 1
sslcacert = /etc/rhsm/ca/redhat-uep.pem
sslclientkey = /etc/pki/entitlement/1234567890-key.pem
sslclientcert = /etc/pki/entitlement/1234567890.pem
metadata_expire = 86400
enabled_metadata = 0

[fluffy-snake-tools-1-rpms]
name = Red Hat JBoss Core Services Text-Only Advisories
baseurl = http://cdn.foo.com/content/dist/snakes/1.0/$basearch/os
enabled = 1
gpgcheck = 1
gpgkey = file://
sslverify = 1
sslcacert = /etc/rhsm/ca/redhat-uep.pem
sslclientkey = /etc/pki/entitlement/1234567890-key.pem
sslclientcert = /etc/pki/entitlement/1234567890.pem
metadata_expire = 86400
enabled_metadata = 0
"""


class TestEnabledReposProfile(unittest.TestCase):
    """
    Test case of EnabledReposProfile class
    """

    def setUp(self) -> None:
        # Mock dnf module
        dnf_patcher = patch("rhsm.profile.dnf")
        self.dnf_mock = dnf_patcher.start()
        mock_db = mock.Mock()
        mock_db.conf = mock.Mock()
        mock_db.conf.substitutions = {"releasever": "1", "basearch": "x86_64"}
        self.dnf_mock.dnf.Base = mock.Mock(return_value=mock_db)
        self.addCleanup(dnf_patcher.stop)

    def test_enabled_repos(self):
        """
        Test the case, when there is one enabled repository
        """
        with tempfile.NamedTemporaryFile() as tmp_repo_file:
            tmp_repo_file.write(bytes(REPO_FILE_CONTENT, encoding="utf-8"))
            tmp_repo_file.flush()
            enabled_repos = EnabledReposProfile(tmp_repo_file.name)
            repo_list = enabled_repos.collect()
            self.assertEqual(len(repo_list), 1)
            self.assertEqual(repo_list[0]["repositoryid"], "fluffy-snake-tools-1-rpms")
            self.assertEqual(
                repo_list[0]["baseurl"], ["http://cdn.foo.com/content/dist/snakes/1.0/x86_64/os"]
            )



# Make argparse friendlier to i18n/l10n
#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
Make argparse friendlier to i18n/l10n

Just use this instead of argparse, the interface should be the same.

For some backgorund, see:
http://bugs.python.org/issue4319
"""
import argparse
from argparse import ArgumentParser as _ArgumentParser
import sys

from subscription_manager.i18n import ugettext as _


argparse._ = _

# note default is lower caps
USAGE = _("%(prog)s [OPTIONS]")


class ArgumentParser(_ArgumentParser):
    def print_help(self):
        sys.stdout.write(self.format_help())

    def error(self, msg):
        """
        Override default error handler to localize

        prints command usage, then the error string, and exits.
        """
        self.print_usage(sys.stderr)
        # translators: arg 1 is the program name, arg 2 is the error message
        print((_("{prog}: error: {msg}")).format(prog=self.prog, msg=msg))
        self.exit(2)



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
from subscription_manager.cli_command.cli import CliCommand
from subscription_manager.i18n import ugettext as _

SM = "subscription-manager"


class PluginsCommand(CliCommand):
    def __init__(self):
        shortdesc = _("View and configure with 'subscription-manager plugins'")
        super(PluginsCommand, self).__init__("plugins", shortdesc, False)

        self.parser.add_argument(
            "--list",
            action="store_true",
            help=_("list {SM} plugins").format(SM=SM),
        )
        self.parser.add_argument(
            "--listslots", action="store_true", help=_("list {SM} plugin slots").format(SM=SM)
        )
        self.parser.add_argument(
            "--listhooks", action="store_true", help=_("list {SM} plugin hooks").format(SM=SM)
        )
        self.parser.add_argument(
            "--verbose", action="store_true", default=False, help=_("show verbose plugin info")
        )

    def _validate_options(self):
        # default to list
        if not (self.options.list or self.options.listslots or self.options.listhooks):
            self.options.list = True

    def require_connection(self):
        return False

    def _list_plugins(self):
        for plugin_class in list(self.plugin_manager.get_plugins().values()):
            enabled = _("disabled")
            if plugin_class.conf.is_plugin_enabled():
                enabled = _("enabled")
            print("{key}: {enabled}".format(key=plugin_class.get_plugin_key(), enabled=enabled))
            if self.options.verbose:
                print(plugin_class.conf)

    def _do_command(self):
        self._validate_options()

        if self.options.list:
            self._list_plugins()

        if self.options.listslots:
            for slot in self.plugin_manager.get_slots():
                print(slot)

        if self.options.listhooks:
            # get_slots is nicely sorted for presentation
            for slot in self.plugin_manager.get_slots():
                print(slot)
                for hook in sorted(self.plugin_manager._slot_to_funcs[slot], key=lambda func: func.__name__):
                    hook_key = hook.__self__.__class__.get_plugin_key()
                    print("\t{key}.{name}".format(key=hook_key, name=hook.__name__))



from ..test_managercli import TestCliProxyCommand
from subscription_manager import managercli


class TestIdentityCommand(TestCliProxyCommand):
    command_class = managercli.IdentityCommand

    def test_regenerate_no_force(self):
        self.cc.main(["--regenerate"])

    def test_token_no_force(self):
        self._test_exception(["--token", "eyJhbGciOiJSUzI1NiIsInR5cCIg"])

    def test_token_with_force(self):
        self._test_no_exception(["--regenerate", "--token", "eyJhbGciOiJSUzI1NiIsInR5cCIg", "--force"])



# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
__all__ = [
    "NAME_BASE",
    "VERSION",
    "BUS_NAME",
    "INTERFACE_BASE",
    "ROOT_DBUS_PATH",
    "MAIN_INTERFACE",
    "MAIN_DBUS_PATH",
    "REGISTER_INTERFACE",
    "REGISTER_DBUS_PATH",
    "UNREGISTER_INTERFACE",
    "UNREGISTER_DBUS_PATH",
    "CONFIG_INTERFACE",
    "CONFIG_DBUS_PATH",
    "ATTACH_INTERFACE",
    "ATTACH_DBUS_PATH",
    "PRODUCTS_INTERFACE",
    "PRODUCTS_DBUS_PATH",
    "ENTITLEMENT_INTERFACE",
    "ENTITLEMENT_DBUS_PATH",
    "CONSUMER_INTERFACE",
    "CONSUMER_DBUS_PATH",
    "SYSPURPOSE_INTERFACE",
    "SYSPURPOSE_DBUS_PATH",
    "DBUS_PROPERTIES_INTERFACE",
]

# The base of the 'well known name' used for bus and service names, as well
# as interface names and object paths.
#
# "com.redhat.RHSM1"
NAME_BASE = "com.redhat.RHSM"
VERSION = "1"
BUS_NAME = NAME_BASE + VERSION

# The default interface name for objects we share on this service.
INTERFACE_BASE = BUS_NAME

# The root of the objectpath tree for our services.
# Note: No trailing '/'
#
# /com/redhat/RHSM1
ROOT_DBUS_PATH = "/" + str.replace(BUS_NAME, ".", "/")

MAIN_INTERFACE = INTERFACE_BASE
MAIN_DBUS_PATH = ROOT_DBUS_PATH

REGISTER_INTERFACE = "%s.%s" % (INTERFACE_BASE, "RegisterServer")
REGISTER_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "RegisterServer")

PRIVATE_REGISTER_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Register")
PRIVATE_REGISTER_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Register")

UNREGISTER_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Unregister")
UNREGISTER_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Unregister")

CONFIG_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Config")
CONFIG_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Config")

ATTACH_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Attach")
ATTACH_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Attach")

PRODUCTS_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Products")
PRODUCTS_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Products")

ENTITLEMENT_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Entitlement")
ENTITLEMENT_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Entitlement")

CONSUMER_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Consumer")
CONSUMER_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Consumer")

SYSPURPOSE_INTERFACE = "%s.%s" % (INTERFACE_BASE, "Syspurpose")
SYSPURPOSE_DBUS_PATH = "%s/%s" % (ROOT_DBUS_PATH, "Syspurpose")

DBUS_PROPERTIES_INTERFACE = "org.freedesktop.DBus.Properties"



# Copyright (c) 2020 ATIX AG
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import subprocess
from typing import Dict, List, Union

from rhsmlib.facts import collector

log = logging.getLogger(__name__)


class SupportedArchesCollector(collector.FactsCollector):
    """
    Class used for collecting packages architectures of a host
    """

    DEBIAN_DISTRIBUTIONS: List[str] = ["debian", "ubuntu"]

    def __init__(
        self,
        arch: str = None,
        prefix: str = None,
        testing: bool = None,
        collected_hw_info: Dict[str, Union[str, int, bool, None]] = None,
    ):
        super(SupportedArchesCollector, self).__init__(
            arch=arch, prefix=prefix, testing=testing, collected_hw_info=collected_hw_info
        )

    def get_arches_on_debian(self) -> Dict[str, str]:
        """
        Try to return content of all supported packages architectures
        :return: dictionary containing architectures
        Otherwise, a dictionary with an empty string for 'supported_architectures' is returned.
        """
        arches: List[str] = []

        try:
            arch: str = subprocess.check_output(["dpkg", "--print-architecture"]).decode("UTF-8")
            if arch != "":
                arches.append(arch.rstrip("\n"))
        except Exception as e:
            log.error("Error getting dpkg main architecture: %s", e)

        try:
            arch: str = subprocess.check_output(["dpkg", "--print-foreign-architectures"]).decode("UTF-8")
            if arch != "":
                arches.append(arch.rstrip("\n"))
        except Exception as e:
            log.error("Error getting dpkg foreign architecture: %s", e)

        return {"supported_architectures": ",".join(arches)}

    def get_all(self) -> Dict[str, str]:
        """
        Get all architectures of a debian / ubuntu host
        :return: dictionary containing architectures
        """
        arch_info: Dict[str, str] = {}

        dist_name: str = self._collected_hw_info["distribution.name"].lower()
        if any(os in dist_name for os in self.DEBIAN_DISTRIBUTIONS):
            arch_info = self.get_arches_on_debian()

        return arch_info



# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
from unittest import mock
import json
import datetime

from rhsmlib.dbus.objects import ProductsDBusObject

from test.rhsmlib.base import DBusServerStubProvider


START_DATE = datetime.datetime.now() - datetime.timedelta(days=100)
NOW_DATE = datetime.datetime.now()
END_DATE = datetime.datetime.now() + datetime.timedelta(days=265)


class TestProductsDBusObject(DBusServerStubProvider):
    dbus_class = ProductsDBusObject
    dbus_class_kwargs = {}

    @classmethod
    def setUpClass(cls) -> None:
        list_patch = mock.patch(
            "rhsmlib.dbus.objects.products.InstalledProducts.list",
            name="list",
        )
        cls.patches["list"] = list_patch.start()
        cls.addClassCleanup(list_patch.stop)

        super().setUpClass()

    def test_ListInstalledProducts__no_filter(self):
        expected = [
            (
                "Red Hat Enterprise Linux Server",
                "69",
                "7.4",
                "x86_64",
                "subscribed",
                [],
                START_DATE.strftime("%Y-%m-%d"),
                END_DATE.strftime("%Y-%m-%d"),
            ),
            (
                "Red Hat Enterprise Linux Server - Extended Update Support",
                "70",
                "7.2",
                "x86_64",
                "subscribed",
                [],
                START_DATE.strftime("%Y-%m-%d"),
                END_DATE.strftime("%Y-%m-%d"),
            ),
        ]

        self.patches["list"].return_value = expected

        result = self.obj.ListInstalledProducts.__wrapped__(self.obj, "", {}, self.LOCALE)
        self.assertEqual(json.dumps(expected), result)



# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from collections import deque
import os
import unittest

from rhsm.bitstream import GhettoBitStream
from rhsm.huffman import HuffmanNode
from rhsm.pathtree import PathTree, PATH_END

DATA = os.path.join(os.path.dirname(os.path.abspath(__file__)), "entitlement_data.bin")


class TestPathTree(unittest.TestCase):
    def test_get_leaf_from_dict(self):
        codes = {"1010": "abc"}
        bitstream = "10101111110000"
        ret = PathTree._get_leaf_from_dict(codes, bitstream)
        self.assertEqual(ret, "abc")

    # see v3 entitlement cert format docs for explanation of how node count
    # is represented, which will explain the following tests

    def test_get_node_count_small(self):
        bs = GhettoBitStream([])
        bs.bytes = deque([6])
        ret = PathTree._get_node_count(bs)
        self.assertEqual(ret, 6)

    def test_get_node_count_medium(self):
        bs = GhettoBitStream([])
        # count bigger than 127, only need 1 byte to represent it
        bs.bytes = deque([129, 150])
        ret = PathTree._get_node_count(bs)
        self.assertEqual(ret, 150)

    def test_get_node_count_big(self):
        bs = GhettoBitStream([])
        # count bigger than 127, need next 2 bytes to represent it
        bs.bytes = deque([130, 1, 17])
        ret = PathTree._get_node_count(bs)
        self.assertEqual(ret, 273)

    def test_unpack_data(self):
        data = open(DATA, "rb").read()
        nodes, bits = PathTree._unpack_data(data)
        self.assertEqual(len(nodes), 6)
        # first node always gets weight of 1
        self.assertEqual(nodes[0].weight, 1)
        self.assertEqual(nodes[0].value, "never")
        self.assertEqual(nodes[5].weight, 6)
        self.assertEqual(nodes[5].value, "")
        self.assertEqual(len(bits), 6)

    def test_generate_path_leaves(self):
        data = open(DATA, "rb").read()
        nodes, bits = PathTree._unpack_data(data)
        ret = PathTree._generate_path_leaves(GhettoBitStream(bits))

        self.assertEqual(len(ret), 4)
        for node in ret:
            self.assertTrue(isinstance(node, HuffmanNode))

    def test_generate_path_tree(self):
        data = open(DATA, "rb").read()
        pt = PathTree(data).path_tree
        self.assertTrue("foo" in pt)
        self.assertEqual(len(list(pt.keys())), 1)

    def test_match_path(self):
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        self.assertTrue(pt.match_path("/foo/path"))
        self.assertTrue(pt.match_path("/foo/path/"))
        # the '2' should match against "$releasever"
        self.assertTrue(pt.match_path("/foo/path/always/2"))
        self.assertTrue(pt.match_path("/foo/path/bar"))
        self.assertTrue(pt.match_path("/foo/path/bar/a/b/c"))
        self.assertFalse(pt.match_path("/foo"))
        self.assertFalse(pt.match_path("/bar"))

    def test_build_path_list(self):
        f = os.path.join(os.path.dirname(os.path.abspath(__file__)), "satellite_generated_data.bin")
        data = open(f, "rb").read()
        pt = PathTree(data)
        paths = []
        pt.build_path_list(paths)
        expected_stubs = [
            "/$releasever/$basearch/os",
            "/$releasever/$basearch/optional/os",
            "/$releasever/$basearch/rh-common/os",
            "/$releasever/$basearch/rhn-tools/os",
            "/7Server/$basearch/extras/os",
            "/7Server/$basearch/sat-tools/6.3/os",
        ]
        expected = [
            "/DPS_Satellite/Library/WF-RHEL-7-CV-2018_33-OS/content/dist/rhel/server/7" + stub
            for stub in expected_stubs
        ]
        # Assert the lists contain the same items regardless of order
        self.assertCountEqual(expected, paths)

    def test_match_path_listing(self):
        tree = {"foo": [{"path": [{"bar": [{PATH_END: None}]}]}]}
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        pt.path_tree = tree
        self.assertTrue(pt.match_path("/foo/path/bar/listing"))
        self.assertTrue(pt.match_path("/foo/path/listing"))
        self.assertTrue(pt.match_path("/foo/listing"))
        self.assertFalse(pt.match_path("/foo/path/alfred"))
        self.assertFalse(pt.match_path("/foo/path/listing/for/alfred"))

    def test_match_variable(self):
        tree = {"foo": [{"$releasever": [{"bar": [{PATH_END: None}]}]}]}
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        # just swap out the pre-cooked data with out with
        pt.path_tree = tree
        self.assertTrue(pt.match_path("/foo/path/bar"))
        self.assertFalse(pt.match_path("/foo/path/abc"))

    def test_match_first_variable(self):
        tree = {"$anything": [{"$releasever": [{"bar": [{PATH_END: None}]}]}]}
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        # just swap out the pre-cooked data with out with
        pt.path_tree = tree
        self.assertTrue(pt.match_path("/foo/path/bar"))
        self.assertFalse(pt.match_path("/foo/path/abc"))

    def test_match_last_variable(self):
        tree = {"foo": [{"$releasever": [{"$bar": [{PATH_END: None}]}]}]}
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        # just swap out the pre-cooked data with out with
        pt.path_tree = tree
        self.assertTrue(pt.match_path("/foo/path/bar"))
        self.assertTrue(pt.match_path("/foo/path/abc"))
        self.assertFalse(pt.match_path("/boo/path/abc"))

    def test_match_different_variables(self):
        tree1 = {
            "foo": [{"$releasever": [{"bar": [{PATH_END: None}]}], "jarjar": [{"binks": [{PATH_END: None}]}]}]
        }
        tree2 = {
            "foo": [{"jarjar": [{"binks": [{PATH_END: None}]}], "$releasever": [{"bar": [{PATH_END: None}]}]}]
        }
        tree3 = {
            "foo": [
                {"$releasever": [{"bar": [{PATH_END: None}]}]},
                {"jarjar": [{"binks": [{PATH_END: None}]}]},
            ]
        }
        tree4 = {
            "foo": [
                {"jarjar": [{"binks": [{PATH_END: None}]}]},
                {"$releasever": [{"bar": [{PATH_END: None}]}]},
            ]
        }
        trees = [tree1, tree2, tree3, tree4]
        data = open(DATA, "rb").read()
        pt = PathTree(data)
        # just swap out the pre-cooked data with out with
        for tree in trees:
            pt.path_tree = tree
            self.assertTrue(pt.match_path("/foo/path/bar"))
            self.assertFalse(pt.match_path("/foo/path/abc"))
            self.assertFalse(pt.match_path("/foo/path/abc"))
            self.assertTrue(pt.match_path("/foo/jarjar/binks"))
            self.assertTrue(pt.match_path("/foo/jarjar/bar"))
            self.assertFalse(pt.match_path("/foo/jarjar/notbinks"))



# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from unittest import mock
import json

import subscription_manager.injection as inj

from subscription_manager.cache import InstalledProductsManager, ContentAccessModeCache
from subscription_manager.cp_provider import CPProvider
from subscription_manager.facts import Facts
from subscription_manager.identity import Identity
from subscription_manager.plugins import PluginManager

from test.fixture import set_up_mock_sp_store

from test.rhsmlib.base import InjectionMockingTest

from rhsm import connection

from rhsmlib.services import register, exceptions

CONSUMER_CONTENT_JSON = """{"hypervisorId": null,
        "serviceLevel": "",
        "autoheal": true,
        "idCert": {
          "key": "FAKE_KEY",
          "cert": "FAKE_CERT",
          "serial" : {
            "id" : 5196045143213189102,
            "revoked" : false,
            "collected" : false,
            "expiration" : "2033-04-25T18:03:06+0000",
            "serial" : 5196045143213189102,
            "created" : "2017-04-25T18:03:06+0000",
            "updated" : "2017-04-25T18:03:06+0000"
          },
          "id" : "8a8d011e5ba64700015ba647fbd20b88",
          "created" : "2017-04-25T18:03:07+0000",
          "updated" : "2017-04-25T18:03:07+0000"
        },
        "owner": {
          "href": "/owners/admin",
          "displayName": "Admin Owner",
          "id": "ff808081550d997c01550d9adaf40003",
          "key": "admin",
          "contentAccessMode": "entitlement"
        },
        "href": "/consumers/c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "facts": {}, "id": "ff808081550d997c015511b0406d1065",
        "uuid": "c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "guestIds": null, "capabilities": null,
        "environment": null, "installedProducts": null,
        "canActivate": false, "type": {"manifest": false,
        "id": "1000", "label": "system"}, "annotations": null,
        "username": "admin", "updated": "2016-06-02T15:16:51+0000",
        "lastCheckin": null, "entitlementCount": 0, "releaseVer":
        {"releaseVer": null}, "entitlementStatus": "valid", "name":
        "test.example.com", "created": "2016-06-02T15:16:51+0000",
        "contentTags": null, "dev": false}"""

# Following consumer do not contain information about content access mode
OLD_CONSUMER_CONTENT_JSON = """{"hypervisorId": null,
        "serviceLevel": "",
        "autoheal": true,
        "idCert": {
          "key": "FAKE_KEY",
          "cert": "FAKE_CERT",
          "serial" : {
            "id" : 5196045143213189102,
            "revoked" : false,
            "collected" : false,
            "expiration" : "2033-04-25T18:03:06+0000",
            "serial" : 5196045143213189102,
            "created" : "2017-04-25T18:03:06+0000",
            "updated" : "2017-04-25T18:03:06+0000"
          },
          "id" : "8a8d011e5ba64700015ba647fbd20b88",
          "created" : "2017-04-25T18:03:07+0000",
          "updated" : "2017-04-25T18:03:07+0000"
        },
        "owner": {
          "href": "/owners/admin",
          "displayName": "Admin Owner",
          "id": "ff808081550d997c01550d9adaf40003",
          "key": "admin"
        },
        "href": "/consumers/c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "facts": {}, "id": "ff808081550d997c015511b0406d1065",
        "uuid": "c1b8648c-6f0a-4aa5-b34e-b9e62c0e4364",
        "guestIds": null, "capabilities": null,
        "environments": null, "installedProducts": null,
        "canActivate": false, "type": {"manifest": false,
        "id": "1000", "label": "system"}, "annotations": null,
        "username": "admin", "updated": "2016-06-02T15:16:51+0000",
        "lastCheckin": null, "entitlementCount": 0, "releaseVer":
        {"releaseVer": null}, "entitlementStatus": "valid", "name":
        "test.example.com", "created": "2016-06-02T15:16:51+0000",
        "contentTags": null, "dev": false}"""

OWNERS_CONTENT_JSON = """[
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Donald Duck",
        "href": "/owners/donaldduck",
        "id": "ff80808170523d030170523d34890003",
        "key": "donaldduck",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    },
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Admin Owner",
        "href": "/owners/admin",
        "id": "ff80808170523d030170523d347c0002",
        "key": "admin",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    },
    {
        "autobindDisabled": false,
        "autobindHypervisorDisabled": false,
        "contentAccessMode": "entitlement",
        "contentAccessModeList": "entitlement,org_environment",
        "contentPrefix": null,
        "created": "2020-02-17T08:21:47+0000",
        "defaultServiceLevel": null,
        "displayName": "Snow White",
        "href": "/owners/snowwhite",
        "id": "ff80808170523d030170523d348a0004",
        "key": "snowwhite",
        "lastRefreshed": null,
        "logLevel": null,
        "parentOwner": null,
        "updated": "2020-02-17T08:21:47+0000",
        "upstreamConsumer": null
    }
]
"""


class RegisterServiceTest(InjectionMockingTest):
    def setUp(self):
        super(RegisterServiceTest, self).setUp()
        self.mock_identity = mock.Mock(spec=Identity, name="Identity")
        self.mock_cp = mock.Mock(spec=connection.UEPConnection, name="UEPConnection")

        # Mock a basic auth connection
        self.mock_cp.username = "username"
        self.mock_cp.password = "password"

        # Add a mock cp_provider
        self.mock_cp_provider = mock.Mock(spec=CPProvider, name="CPProvider")

        # Add a mock for content access mode cache
        self.mock_content_access_mode_cache = mock.Mock(
            spec=ContentAccessModeCache, name="ContentAccessModeCache"
        )
        self.mock_content_access_mode_cache.read_data = mock.Mock(return_value="entitlement")

        # For the tests in which it's used, the consumer_auth cp and basic_auth cp can be the same
        self.mock_cp_provider.get_consumer_auth_cp.return_value = self.mock_cp
        self.mock_cp_provider.get_basic_auth_cp.return_value = self.mock_cp

        self.mock_pm = mock.Mock(spec=PluginManager, name="PluginManager")
        self.mock_installed_products = mock.Mock(
            spec=InstalledProductsManager, name="InstalledProductsManager"
        )
        self.mock_facts = mock.Mock(spec=Facts, name="Facts")
        self.mock_facts.get_facts.return_value = {}

        syspurpose_patch = mock.patch("subscription_manager.syspurposelib.SyncedStore")
        self.mock_sp_store = syspurpose_patch.start()
        self.mock_sp_store, self.mock_sp_store_contents = set_up_mock_sp_store(self.mock_sp_store)
        self.addCleanup(syspurpose_patch.stop)

        self.mock_syspurpose = mock.Mock()

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            return self.mock_identity
        elif args[0] == inj.PLUGIN_MANAGER:
            return self.mock_pm
        elif args[0] == inj.INSTALLED_PRODUCTS_MANAGER:
            return self.mock_installed_products
        elif args[0] == inj.FACTS:
            return self.mock_facts
        elif args[0] == inj.CP_PROVIDER:
            return self.mock_cp_provider
        elif args[0] == inj.CONTENT_ACCESS_MODE_CACHE:
            return self.mock_content_access_mode_cache
        else:
            return None

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_normally(self, mock_persist_consumer, mock_write_cache):
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", environments="environment")

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environments="environment",
            keys=None,
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            consumer_type="system",
            role="",
            addons=[],
            service_level="",
            usage="",
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_normally_old_candlepin(self, mock_persist_consumer, mock_write_cache):
        """
        Test for the case, when candlepin server returns consumer without information about
        content access mode.
        """
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        expected_consumer = json.loads(OLD_CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        consumer = register_service.register("org", name="name", environments="environment")

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environments="environment",
            keys=None,
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            consumer_type="system",
            role="",
            addons=[],
            service_level="",
            usage="",
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)
        assert "owner" in consumer
        assert "contentAccessMode" in consumer["owner"]
        assert "entitlement" == consumer["owner"]["contentAccessMode"]

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_normally_no_owner(self, mock_persist_consumer, mock_write_cache):
        """
        Test for the case, when candlepin server returns consumer without owner
        """
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        expected_consumer = json.loads(OLD_CONSUMER_CONTENT_JSON)
        del expected_consumer["owner"]
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        consumer = register_service.register("org", name="name", environments="environment")

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environments="environment",
            keys=None,
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            consumer_type="system",
            role="",
            addons=[],
            service_level="",
            usage="",
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)
        assert "owner" not in consumer

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_normally_with_no_org_specified(self, mock_persist_consumer, mock_write_cache):
        """
        This test is intended for the case, when no organization is specified, but user is member
        only of one organization and thus it can be automatically selected
        """
        self.mock_cp.getOwnerList = mock.Mock()
        self.mock_cp.getOwnerList.return_value = [
            {
                "created": "2020-08-18T07:57:47+0000",
                "updated": "2020-08-18T07:57:47+0000",
                "id": "ff808081740092cd01740092fe540002",
                "key": "snowwhite",
                "displayName": "Snow White",
                "parentOwner": None,
                "contentPrefix": None,
                "defaultServiceLevel": None,
                "upstreamConsumer": None,
                "logLevel": None,
                "autobindDisabled": False,
                "autobindHypervisorDisabled": False,
                "contentAccessMode": "entitlement",
                "contentAccessModeList": "entitlement,org_environment",
                "lastRefreshed": None,
                "href": "/owners/snowwhite",
            }
        ]
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)

        def _get_owner_cb(orgs):
            pass

        def _no_owner_cb(username):
            pass

        org = register_service.determine_owner_key(
            username=self.mock_cp.username, get_owner_cb=_get_owner_cb, no_owner_cb=_no_owner_cb
        )

        self.assertIsNotNone(org)

        register_service.register(org, name="name", environments="environment")

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="snowwhite",
            environments="environment",
            keys=None,
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            consumer_type="system",
            role="",
            addons=[],
            service_level="",
            usage="",
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_with_activation_keys(self, mock_persist_consumer, mock_write_cache):
        self.mock_cp.username = None
        self.mock_cp.password = None
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", activation_keys=[1])

        self.mock_cp.registerConsumer.assert_called_once_with(
            name="name",
            facts={},
            owner="org",
            environments=None,
            keys=[1],
            installed_products=[],
            jwt_token=None,
            content_tags=[],
            consumer_type="system",
            role="",
            addons=[],
            service_level="",
            usage="",
        )
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_register_with_consumerid(self, mock_persist_consumer, mock_write_cache):
        self.mock_identity.is_valid.return_value = False
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.getConsumer.return_value = json.loads(CONSUMER_CONTENT_JSON)

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name", consumerid="consumerid")

        self.mock_cp.getConsumer.assert_called_once_with("consumerid")
        self.mock_installed_products.write_cache.assert_called()

        mock_persist_consumer.assert_called_once_with(expected_consumer)
        mock_write_cache.assert_called_once()
        expected_plugin_calls = [
            mock.call("pre_register_consumer", name="name", facts={}),
            mock.call("post_register_consumer", consumer=expected_consumer, facts={}),
        ]
        self.assertEqual(expected_plugin_calls, self.mock_pm.run.call_args_list)

    def _build_options(self, activation_keys=None, environments=None, force=None, name=None, consumerid=None):
        return {
            "activation_keys": activation_keys,
            "environments": environments,
            "force": force,
            "name": name,
            "consumerid": consumerid,
        }

    def test_fails_when_previously_registered(self):
        self.mock_identity.is_valid.return_value = True

        with self.assertRaisesRegex(exceptions.ValidationError, r".*system is already registered.*"):
            register.RegisterService(self.mock_cp).validate_options(self._build_options())

    def test_allows_force(self):
        self.mock_identity.is_valid.return_value = True
        options = self._build_options(force=True)
        register.RegisterService(self.mock_cp).validate_options(options)

    @mock.patch("rhsmlib.services.register.syspurposelib.write_syspurpose_cache", return_value=True)
    @mock.patch("rhsmlib.services.register.managerlib.persist_consumer_cert")
    def test_reads_syspurpose(self, mock_persist_consumer, mock_write_cache):
        self.mock_installed_products.format_for_server.return_value = []
        self.mock_installed_products.tags = []
        self.mock_identity.is_valid.return_value = False
        self.mock_sp_store_contents["role"] = "test_role"
        self.mock_sp_store_contents["service_level_agreement"] = "test_sla"
        self.mock_sp_store_contents["addons"] = ["addon1"]
        self.mock_sp_store_contents["usage"] = "test_usage"

        expected_consumer = json.loads(CONSUMER_CONTENT_JSON)
        self.mock_cp.registerConsumer.return_value = expected_consumer

        register_service = register.RegisterService(self.mock_cp)
        register_service.register("org", name="name")

        self.mock_cp.registerConsumer.assert_called_once_with(
            addons=["addon1"],
            content_tags=[],
            environments=None,
            facts={},
            installed_products=[],
            jwt_token=None,
            keys=None,
            name="name",
            owner="org",
            role="test_role",
            service_level="test_sla",
            consumer_type="system",
            usage="test_usage",
        )
        mock_write_cache.assert_called_once()

    def test_does_not_require_basic_auth_with_activation_keys(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1])
        register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_basic_auth_with_activation_keys(self):
        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1])
        with self.assertRaisesRegex(exceptions.ValidationError, r".*do not require user credentials.*"):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_environment_with_activation_keys(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1], environments="environment")
        with self.assertRaisesRegex(exceptions.ValidationError, r".*do not allow environments.*"):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_does_not_allow_environment_with_consumerid(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(activation_keys=[1], consumerid="consumerid")
        with self.assertRaisesRegex(exceptions.ValidationError, r".*previously registered.*"):
            register.RegisterService(self.mock_cp).validate_options(options)

    def test_requires_basic_auth_for_normal_registration(self):
        self.mock_cp.username = None
        self.mock_cp.password = None

        self.mock_identity.is_valid.return_value = False
        options = self._build_options(consumerid="consumerid")
        with self.assertRaisesRegex(exceptions.ValidationError, r".*Missing username.*"):
            register.RegisterService(self.mock_cp).validate_options(options)



from .fixture import SubManFixture
from subscription_manager.overrides import Overrides, Override


class OverrideTests(SubManFixture):
    def setUp(self):
        SubManFixture.setUp(self)
        self.overrides = Overrides()

    def test_add_function(self):
        repos = ["x", "y"]
        override_props = {"a": "b", "c": "d"}
        overrides = [
            Override(repo, name, value) for repo in repos for name, value in list(override_props.items())
        ]
        expected = [
            {"contentLabel": "x", "name": "a", "value": "b"},
            {"contentLabel": "x", "name": "c", "value": "d"},
            {"contentLabel": "y", "name": "a", "value": "b"},
            {"contentLabel": "y", "name": "c", "value": "d"},
        ]
        result = self.overrides._add(overrides)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_function(self):
        repos = ["x", "y"]
        props_to_remove = ["a", "b"]
        removes = [Override(repo, name) for repo in repos for name in props_to_remove]
        expected = [
            {"contentLabel": "x", "name": "a"},
            {"contentLabel": "x", "name": "b"},
            {"contentLabel": "y", "name": "a"},
            {"contentLabel": "y", "name": "b"},
        ]
        result = self.overrides._remove(removes)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_all(self):
        repos = ["x", "y"]
        expected = [
            {"contentLabel": "x"},
            {"contentLabel": "y"},
        ]
        result = self.overrides._remove_all(repos)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_all_with_no_repos_given(self):
        repos = []
        result = self.overrides._remove_all(repos)
        self.assertEqual(None, result)



# Copyright (c) 2022 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

"""
This module contains a minimal ad-hoc parser for the output of `dmidecode`.

Tailored for the rest of the subscription-manager code, not general enough.
"""

import collections
import contextlib
import enum
import logging
import os
import re
import shutil
import subprocess
from typing import Dict, List, Union, Optional

log = logging.getLogger(__name__)


class DmidecodeParser:
    """
    Simple parser for the dmidecode output.

    This class provides a simple way to parse the output of the dmidecode(1)
    tool, either by running dmidecode(1) directly, or by reading its output
    from a text file.

    This parser only parses the output and collects the various entries,
    so they can be queried as needed.
    """

    @enum.unique
    class DmiTypes(enum.Enum):
        """
        The known DMI types.

        The values represent the actual values in the SMBIOS specification,
        so it possible to use this enum to avoid specifying them when
        looking up sections.
        """

        BIOS_INFORMATION = 0
        SYSTEM_INFORMATION = 1
        BASEBOARD_INFORMATION = 2
        SYSTEM_ENCLOSURE_OR_CHASSIS = 3
        PROCESSOR_INFORMATION = 4
        MEMORY_CONTROLLER_INFORMATION = 5
        MEMORY_MODULE_INFORMATION = 6
        CACHE_INFORMATION = 7
        PORT_CONNECTOR_INFORMATION = 8
        SYSTEM_SLOTS = 9
        ON_BOARD_DEVICES_INFORMATION = 10
        OEM_STRINGS = 11
        SYSTEM_CONFIGURATION_OPTIONS = 12
        BIOS_LANGUAGE_INFORMATION = 13
        GROUP_ASSOCIATIONS = 14
        SYSTEM_EVENT_LOG = 15
        PHYSICAL_MEMORY_ARRAY = 16
        MEMORY_DEVICE = 17
        THIRTYTWO_BIT_MEMORY_ERROR_INFORMATION = 18
        MEMORY_ARRAY_MAPPED_ADDRESS = 19
        MEMORY_DEVICE_MAPPED_ADDRESS = 20
        BUILT_IN_POINTING_DEVICE = 21
        PORTABLE_BATTERY = 22
        SYSTEM_RESET = 23
        HARDWARE_SECURITY = 24
        SYSTEM_POWER_CONTROLS = 25
        VOLTAGE_PROBE = 26
        COOLING_DEVICE = 27
        TEMPERATURE_PROBE = 28
        ELECTRICAL_CURRENT_PROBE = 29
        OUT_OF_BAND_REMOTE_ACCESS = 30
        BOOT_INTEGRITY_SERVICES_ENTRY_POINT = 31
        SYSTEM_BOOT_INFORMATION = 32
        SIXTYFOUR_BIT_MEMORY_ERROR_INFORMATION = 33
        MANAGEMENT_DEVICE = 34
        MANAGEMENT_DEVICE_COMPONENT = 35
        MANAGEMENT_DEVICE_THRESHOLD_DATA = 36
        MEMORY_CHANNEL = 37
        IPMI_DEVICE_INFORMATION = 38
        SYSTEM_POWER_SUPPLY = 39
        ADDITIONAL_INFORMATION = 40
        ONBOARD_DEVICES_EXTENDED_INFORMATION = 41
        MANAGEMENT_CONTROLLER_HOST_INTERFACE = 42
        TPM_DEVICE = 43
        PROCESSOR_ADDITIONAL_INFORMATION = 44

    def __init__(self):
        self._data: Dict[int, Dict[str, Union[str, List[str]]]] = {}
        self._dmi_types = collections.defaultdict(dict)

    def parse(self) -> None:
        """
        Run `dmidecode` and parses its output.

        In case `dmidecode` is not available, cannot be executed, or it exits
        with failure, a warning is logged.
        """
        path: Optional[str] = shutil.which("dmidecode")
        if path is None:
            log.warning("'dmidecode' is not available. No DMI info will be collected.")
            return

        env: Dict[str, str] = dict(os.environ)
        env.update({"LANGUAGE": "en_US.UTF-8"})

        try:
            proc = subprocess.Popen(
                [path], env=env, universal_newlines=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            self._parse_lines(proc.stdout)
        except subprocess.SubprocessError:
            error: str = proc.stderr.read()
            log.warning(f"Error with dmidecode subprocess: {error}")

    def parse_file(self, filename: str) -> None:
        """
        Parse the output of `dmidecode` previously saved into the specified
        file.
        """
        with open(filename, "r") as f:
            self._parse_lines(f)

    def _parse_lines(self, fd) -> None:
        """
        The actual parsing of the `dmidecode` output.

        'fd' is a file object, so anything where it is possible to read line
        by line (using readline()).
        """

        class ParsingState(enum.Enum):
            """
            Helper enum representing the current state in the parsing.
            """

            NONE = enum.auto()  # not in any section
            IN_SECTION = enum.auto()  # within the header of a section
            IN_RECORD = enum.auto()  # within a record of a section
            IN_BLOCK = enum.auto()  # within a block of a record

        def is_value_specified(possible_value: str) -> bool:
            """
            Is a value actually specified/available?

            This is needed because dmidecode prints "Not Specified"/etc
            instead of omitting a value that is not specified as DMI string
            (le sigh).
            """
            return (
                possible_value != "Not Specified"
                and possible_value != "Not Available"
                and possible_value != "Unknown"
                and possible_value != "Unspecified"
            )

        state: ParsingState = ParsingState.NONE
        current_handle: int = None
        current_key: str = None
        value: str
        # regex to parse the start of a section in the output; example:
        #   Handle 0x0000, DMI type 222, 14 bytes
        re_handle: re.Pattern = re.compile(r"^Handle\s+([^,]+),\s+DMI\s+type\s+(\d+),\s+(\d+)\s+bytes$")

        while True:
            # the output of dmidecode is read and parsed line by line;
            # this is done to avoid reading & keeping in memory the whole
            # output, as it can be big (depending on the available hardware,
            # usually)
            line: str = fd.readline()
            if not line:
                break

            line = line.rstrip()
            # empty line: not in a section
            if len(line) == 0:
                # reset all the state variables, and continue with the next
                # line
                state = ParsingState.NONE
                current_handle = None
                current_key = None
                continue

            # this may be the start of a section
            if line.startswith("Handle "):
                m: re.Match = re_handle.fullmatch(line)
                if m:
                    # it really is a section, so get the various details,
                    # and prepare the internal structures for it
                    state = ParsingState.IN_SECTION
                    current_handle = int(m[1], base=16)
                    current_dmi_type = int(m[2])
                    self._data[current_handle] = dict()
                    handles = self._dmi_types[current_dmi_type].get("handles", [])
                    handles.append(current_handle)
                    self._dmi_types[current_dmi_type]["handles"] = handles
                    continue

            # we are in a section, in particular in the beginning of it
            # (after the "Handle: header): the line is the name of the section,
            # so skip it, and assume that records will follow
            if state == ParsingState.IN_SECTION:
                state = ParsingState.IN_RECORD
                continue

            # we are in a record, or in the block of a record: they are handled
            # in a single case because, since we parse line by line, we cannot
            # know when a block ends (and a new record starts)
            if state == ParsingState.IN_RECORD or state == ParsingState.IN_BLOCK:
                # a block
                if line.startswith("\t\t"):
                    if state == ParsingState.IN_RECORD:
                        # had a value, drop it
                        with contextlib.suppress(KeyError):
                            del self._data[current_handle][current_key]
                    value = line[2:]
                    # if the current record had already a value, turn it into
                    # a list, and append the new value to it; this way, each
                    # line in the block will be a new item in the list which
                    # is the value of this record
                    try:
                        current_value = self._data[current_handle][current_key]
                        if not isinstance(current_value, list):
                            current_value = [current_value]
                        current_value.append(value)
                        self._data[current_handle][current_key] = current_value
                    except KeyError:
                        self._data[current_handle][current_key] = value
                    state = ParsingState.IN_BLOCK
                # a record (and not a block, as that is checked earlier)
                elif line.startswith("\t"):
                    # usually a record is a line e.g.
                    #   Foo: value
                    # so split by the first colon, ignoring potentially
                    # wrong lines
                    parts = line[1:].split(":", maxsplit=1)
                    if len(parts) != 2:
                        continue
                    current_key = parts[0]
                    value = parts[1].lstrip()
                    if is_value_specified(value):
                        self._data[current_handle][current_key] = value
                    state = ParsingState.IN_RECORD

    def get_sections(self, dmi_type: Union[int, DmiTypes]) -> List[Dict[str, Union[str, List[str]]]]:
        """
        Get a list of sections for the specified DMI type.

        'dmi_type' can be either an item of the DmiTypes enum, or the integer
        value of a DMI type.
        """
        if isinstance(dmi_type, self.DmiTypes):
            dmi_type = dmi_type.value

        return [self._data[h] for h in self._dmi_types[dmi_type]["handles"]]

    def get_key(self, dmi_type: Union[int, DmiTypes], key: str) -> Union[str, List[str]]:
        """
        Get the value of a specific key of a specified DMI type.

        In case there are more sections of the specified DMI type, this will
        lookup in the first section printed by dmidecode. This function
        is more or less like a convenience wrapper.

        KeyError is raisen if there is no section of the specified DMI type,
        or that section does not have the specified key.
        """
        values: List[Dict[str, Union[str, List[str]]]] = self.get_sections(dmi_type)
        return values[0][key]



# Copyright (c) 2011 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import os
import random
import string
import unittest

from test import subman_marker_functional

from rhsm.connection import (
    ContentConnection,
    UEPConnection,
    BaseRestLib,
    UnauthorizedException,
    ForbiddenException,
    RestlibException,
)
from unittest.mock import patch


def random_string(name, target_length=32):
    '''Returns a len 32 string starting with "name"'''
    for i in range(max(target_length - len(name), 0)):
        name += random.choice(string.ascii_lowercase)
    return name


@subman_marker_functional
class ConnectionTests(unittest.TestCase):
    def setUp(self):
        self.cp = UEPConnection(username="admin", password="admin", insecure=True)

        self.consumer = self.cp.registerConsumer("test-consumer", "system", owner="admin")
        self.consumer_uuid = self.consumer["uuid"]

    def test_supports_resource(self):
        self.assertTrue(self.cp.supports_resource("consumers"))
        self.assertTrue(self.cp.supports_resource("admin"))
        self.assertFalse(self.cp.supports_resource("boogity"))

    def test_has_capability(self):
        self.cp.capabilities = ["cores", "hypervisors_async"]
        self.assertTrue(self.cp.has_capability("cores"))
        self.assertFalse(self.cp.has_capability("boogityboo"))

    def test_update_consumer_can_update_guests_with_empty_list(self):
        self.cp.updateConsumer(self.consumer_uuid, guest_uuids=[])

    def test_update_consumer_can_update_facts_with_empty_dict(self):
        self.cp.updateConsumer(self.consumer_uuid, facts={})

    def test_update_consumer_can_update_installed_products_with_empty_list(self):
        self.cp.updateConsumer(self.consumer_uuid, installed_products=[])

    def test_update_consumer_sets_hypervisor_id(self):
        testing_hypervisor_id = random_string("testHypervisor")
        self.cp.updateConsumer(self.consumer_uuid, hypervisor_id=testing_hypervisor_id)
        hypervisor_id = self.cp.getConsumer(self.consumer_uuid)["hypervisorId"]
        # Hypervisor ID should be set and lower case
        expected = testing_hypervisor_id.lower()
        self.assertEqual(expected, hypervisor_id["hypervisorId"])

    def test_create_consumer_sets_hypervisor_id(self):
        testing_hypervisor_id = random_string("someId")
        consumerInfo = self.cp.registerConsumer(
            "other-test-consumer", "system", owner="admin", hypervisor_id=testing_hypervisor_id
        )
        # Unregister before making assertions, that way it should always happen
        self.cp.unregisterConsumer(consumerInfo["uuid"])
        # Hypervisor ID should be set and lower case
        expected = testing_hypervisor_id.lower()
        self.assertEqual(expected, consumerInfo["hypervisorId"]["hypervisorId"])

    def tearDown(self):
        self.cp.unregisterConsumer(self.consumer_uuid)


@subman_marker_functional
class EntitlementRegenerationTests(unittest.TestCase):
    def setUp(self):
        self.cp = UEPConnection(username="admin", password="admin", insecure=True)

        self.consumer = self.cp.registerConsumer("test-consumer", "system", owner="admin")
        self.consumer_uuid = self.consumer["uuid"]

        entitlements = self.cp.getEntitlementList(self.consumer_uuid)
        self.assertTrue(len(entitlements) > 0)

        self.entitlement = entitlements[0]
        self.entitlement_id = self.entitlement["id"]

    def test_regenerate_entitlements_default(self):
        result = self.cp.regenEntitlementCertificates(self.consumer_uuid)
        self.assertTrue(result)

    def test_regenerate_entitlements_lazy(self):
        result = self.cp.regenEntitlementCertificates(self.consumer_uuid, True)
        self.assertTrue(result)

    def test_regenerate_entitlements_eager(self):
        result = self.cp.regenEntitlementCertificates(self.consumer_uuid, False)
        self.assertTrue(result)

    def test_regenerate_entitlements_bad_uuid(self):
        result = self.cp.regenEntitlementCertificates("bad_consumer_uuid")
        self.assertFalse(result)

    def tearDown(self):
        self.cp.unregisterConsumer(self.consumer_uuid)


@subman_marker_functional
class BindRequestTests(unittest.TestCase):
    def setUp(self):
        self.cp = UEPConnection(username="admin", password="admin", insecure=True)

        consumerInfo = self.cp.registerConsumer("test-consumer", "system", owner="admin")
        self.consumer_uuid = consumerInfo["uuid"]

    @patch.object(BaseRestLib, "validateResult")
    @patch("rhsm.connection.drift_check", return_value=False)
    @patch("httplib.HTTPSConnection", autospec=True)
    def test_bind_no_args(self, mock_conn, mock_drift, mock_validate):

        self.cp.bind(self.consumer_uuid)

        # verify we called request() with kwargs that include 'body' as None
        # Specifically, we are checking that passing in "" to post_request, as
        # it does by default, results in None here. bin() passes no args there
        # so we use the default, "". See  bz #907536
        for (name, args, kwargs) in mock_conn.mock_calls:
            if name == "().request":
                self.assertEqual(None, kwargs["body"])

    @patch.object(BaseRestLib, "validateResult")
    @patch("rhsm.connection.drift_check", return_value=False)
    @patch("httplib.HTTPSConnection", autospec=True)
    def test_bind_by_pool(self, mock_conn, mock_drift, mock_validate):
        # this test is just to verify we make the httplib connection with
        # right args, we don't validate the bind here
        self.cp.bindByEntitlementPool(self.consumer_uuid, "123121111", "1")
        for (name, args, kwargs) in mock_conn.mock_calls:
            if name == "().request":
                self.assertEqual(None, kwargs["body"])


@subman_marker_functional
class ContentConnectionTests(unittest.TestCase):
    def testInsecure(self):
        ContentConnection(host="127.0.0.1", insecure=True)

    # sigh camelCase
    def testEnvProxyUrl(self):
        with patch.dict("os.environ", {"https_proxy": "https://user:pass@example.com:1111"}):
            cc = ContentConnection(host="127.0.0.1")
            self.assertEqual("user", cc.proxy_user)
            self.assertEqual("pass", cc.proxy_password)
            self.assertEqual("example.com", cc.proxy_hostname)
            self.assertEqual(1111, cc.proxy_port)
        assert "https_proxy" not in os.environ

    def testEnvProxyUrlNoPort(self):
        with patch.dict("os.environ", {"https_proxy": "https://user:pass@example.com"}):
            cc = ContentConnection(host="127.0.0.1")
            self.assertEqual("user", cc.proxy_user)
            self.assertEqual("pass", cc.proxy_password)
            self.assertEqual("example.com", cc.proxy_hostname)
            self.assertEqual(3128, cc.proxy_port)
        assert "https_proxy" not in os.environ

    def testEnvProxyUrlNouserOrPass(self):
        with patch.dict("os.environ", {"https_proxy": "https://example.com"}):
            cc = ContentConnection(host="127.0.0.1")
            self.assertEqual(None, cc.proxy_user)
            self.assertEqual(None, cc.proxy_password)
            self.assertEqual("example.com", cc.proxy_hostname)
            self.assertEqual(3128, cc.proxy_port)
        assert "https_proxy" not in os.environ

    def testEnvNoProxy(self):
        with patch.dict("os.environ", {"no_proxy": ".localdomain", "https_proxy": "https://example.com"}):
            cc = ContentConnection(host="localhost.localdomain")
            self.assertEquals(None, cc.proxy_user)
            self.assertEquals(None, cc.proxy_password)
            self.assertEquals(None, cc.proxy_hostname)
            self.assertEquals(None, cc.proxy_port)
        assert "no_proxy" not in os.environ and "https_proxy" not in os.environ

    def testEnvNoProxyWithAsterisk(self):
        with patch.dict("os.environ", {"no_proxy": "*.localdomain", "https_proxy": "https://example.com"}):
            cc = ContentConnection(host="localhost.localdomain")
            self.assertEquals(None, cc.proxy_user)
            self.assertEquals(None, cc.proxy_password)
            self.assertEquals(None, cc.proxy_hostname)
            self.assertEquals(None, cc.proxy_port)
        assert "no_proxy" not in os.environ and "https_proxy" not in os.environ


@subman_marker_functional
class HypervisorCheckinTests(unittest.TestCase):
    def setUp(self):
        self.cp = UEPConnection(username="admin", password="admin", insecure=True)

    def test_hypervisor_checkin_can_pass_empty_map_and_updates_nothing(self):
        response = self.cp.hypervisorCheckIn("admin", "", {})
        if self.cp.has_capability("hypervisors_async"):
            self.assertEqual(response["resultData"], None)
        else:
            self.assertEqual(len(response["failedUpdate"]), 0)
            self.assertEqual(len(response["updated"]), 0)
            self.assertEqual(len(response["created"]), 0)


@subman_marker_functional
class BaseRestLibTests(unittest.TestCase):
    def setUp(self):
        # Get handle to Restlib
        self.conn = UEPConnection().conn
        self.request_type = "GET"
        self.handler = "https://server/path"

    def _validate_response(self, response):
        # wrapper to specify request_type and handler
        return self.conn.validateResult(response, request_type=self.request_type, handler=self.handler)

    def test_invalid_credentitals_thrown_on_401_with_empty_body(self):
        mock_response = {"status": 401}
        self.assertRaises(UnauthorizedException, self._validate_response, mock_response)

    def test_standard_error_handling_on_401_with_defined_body(self):
        self._run_standard_error_handling_test(401)

    def test_standard_error_handling_on_401_with_invalid_json_body(self):
        self._run_standard_error_handling_test_invalid_json(401, UnauthorizedException)

    def test_invalid_credentitals_thrown_on_403_with_empty_body(self):
        mock_response = {"status": 403}
        self.assertRaises(ForbiddenException, self._validate_response, mock_response)

    def test_standard_error_handling_on_403_with_defined_body(self):
        self._run_standard_error_handling_test(403)

    def test_standard_error_handling_on_403_with_invalid_json_body(self):
        self._run_standard_error_handling_test_invalid_json(403, ForbiddenException)

    def _run_standard_error_handling_test_invalid_json(self, expected_error_code, expected_exception):
        mock_response = {"status": expected_error_code, "content": "<this is not valid json>>"}

        self._check_for_remote_server_exception(expected_error_code, expected_exception, mock_response)

    def _run_standard_error_handling_test(self, expected_error):
        expected_error = "My Expected Error."
        mock_response = {"status": expected_error, "content": '{"displayMessage":"%s"}' % expected_error}

        try:
            self._validate_response(mock_response)
            self.fail("An exception should have been thrown.")
        except Exception as ex:
            self.assertTrue(isinstance(ex, RestlibException))
            self.assertEqual(expected_error, ex.code)
            self.assertEqual(expected_error, str(ex))

    def _check_for_remote_server_exception(self, expected_error_code, expected_exception, mock_response):
        try:
            self._validate_response(mock_response)
            self.fail("An %s exception should have been thrown." % expected_exception)
        except Exception as ex:
            self.assertTrue(isinstance(ex, expected_exception))
            self.assertEqual(expected_error_code, ex.code)



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import CliCommand, conf
from subscription_manager.i18n import ugettext as _
from subscription_manager.release import ReleaseBackend, MultipleReleaseProductsError
from subscription_manager.repolib import RepoActionInvoker
from subscription_manager.utils import parse_baseurl_info
import subscription_manager.injection as inj

log = logging.getLogger(__name__)


class ReleaseCommand(CliCommand):
    def __init__(self):
        shortdesc = _("Configure which operating system release to use")
        super(ReleaseCommand, self).__init__("release", shortdesc, True)

        self.parser.add_argument(
            "--show",
            dest="show",
            action="store_true",
            help=_("shows current release setting; default command"),
        )
        self.parser.add_argument(
            "--list",
            dest="list",
            action="store_true",
            help=_("list available releases"),
        )
        self.parser.add_argument(
            "--set",
            dest="release",
            action="store",
            default=None,
            help=_("set the release for this system"),
        )
        self.parser.add_argument(
            "--unset",
            dest="unset",
            action="store_true",
            help=_("unset the release for this system"),
        )

    def _get_consumer_release(self):
        err_msg = _("Error: The 'release' command is not supported by the server.")
        consumer = self.cp.getConsumer(self.identity.uuid)
        if "releaseVer" not in consumer:
            system_exit(os.EX_UNAVAILABLE, err_msg)
        return consumer["releaseVer"]["releaseVer"]

    def show_current_release(self):
        release = self._get_consumer_release()
        if release:
            print(_("Release: {release}").format(release=release))
        else:
            print(_("Release not set"))

    def _do_command(self):
        cdn_url = conf["rhsm"]["baseurl"]
        # note: parse_baseurl_info will populate with defaults if not found
        (cdn_hostname, cdn_port, _cdn_prefix) = parse_baseurl_info(cdn_url)

        # Base CliCommand has already setup proxy info etc
        self.cp_provider.set_content_connection_info(cdn_hostname=cdn_hostname, cdn_port=cdn_port)
        self.release_backend = ReleaseBackend()

        self.assert_should_be_registered()

        repo_action_invoker = RepoActionInvoker()

        if self.options.unset:
            self.cp.updateConsumer(self.identity.uuid, release="")
            inj.require(inj.RELEASE_STATUS_CACHE).delete_cache()
            repo_action_invoker.update()
            print(_("Release preference has been unset"))
        elif self.options.release is not None:
            # get first list of available releases from the server
            try:
                releases = self.release_backend.get_releases()
            except MultipleReleaseProductsError as err:
                log.error("Getting releases failed: {err}".format(err=err))
                system_exit(os.EX_CONFIG, err.translated_message())

            if self.options.release in releases:
                self.cp.updateConsumer(self.identity.uuid, release=self.options.release)
            else:
                system_exit(
                    os.EX_DATAERR,
                    _(
                        "No releases match '{release}'.  " "Consult 'release --list' for a full listing."
                    ).format(release=self.options.release),
                )
            inj.require(inj.RELEASE_STATUS_CACHE).delete_cache()
            repo_action_invoker.update()
            print(_("Release set to: {release}").format(release=self.options.release))
        elif self.options.list:
            try:
                releases = self.release_backend.get_releases()
            except MultipleReleaseProductsError as err:
                log.error("Getting releases failed: {err}".format(err=err))
                system_exit(os.EX_CONFIG, err.translated_message())

            if len(releases) == 0:
                system_exit(os.EX_CONFIG, _("No release versions available, please check subscriptions."))

            print("+-------------------------------------------+")
            print("          {label}       ".format(label=_("Available Releases")))
            print("+-------------------------------------------+")
            for release in releases:
                print(release)

        else:
            self.show_current_release()



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
from rhsmlib.services import unregister

from subscription_manager.action_client import UnregisterActionClient
from subscription_manager.branding import get_branding
from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import CliCommand, conf, ERR_NOT_REGISTERED_CODE, handle_exception
from subscription_manager.i18n import ugettext as _
from subscription_manager.utils import restart_virt_who


class UnRegisterCommand(CliCommand):
    def __init__(self):
        shortdesc = get_branding().CLI_UNREGISTER
        super(UnRegisterCommand, self).__init__("unregister", shortdesc, True)

    def _validate_options(self):
        pass

    def _do_command(self):
        if not self.is_registered():
            # TODO: Should this use the standard NOT_REGISTERED message?
            system_exit(ERR_NOT_REGISTERED_CODE, _("This system is currently not registered."))

        print(
            _("Unregistering from: {hostname}:{port}{prefix}").format(
                hostname=conf["server"]["hostname"],
                port=conf["server"]["port"],
                prefix=conf["server"]["prefix"],
            )
        )
        try:
            unregister.UnregisterService(self.cp).unregister()
        except Exception as e:
            handle_exception("Unregister failed", e)

        # managerlib.unregister reloads the now None provided identity
        # so cp_provider provided auth_cp's should fail, like the below

        # This block is simply to ensure that the yum repos got updated. If it fails,
        # there is no issue since it will most likely be cleaned up elsewhere (most
        # likely by the yum plugin)
        try:
            # there is no consumer cert at this point, a uep object
            # is not useful
            cleanup_certmgr = UnregisterActionClient()
            cleanup_certmgr.update()
        except Exception:
            pass

        self._request_validity_check()

        # We have new credentials, restart virt-who
        restart_virt_who()

        print(_("System has been unregistered."))



from subscription_manager import managercli
from rhsm.certificate2 import CONTENT_ACCESS_CERT_TYPE

from ..stubs import StubConsumerIdentity, StubUEP
from ..fixture import SubManFixture, Capture

from unittest.mock import Mock, patch


MOCK_SERVICE_STATUS_SCA = {
    "status": "Disabled",
    "status_id": "disabled",
    "reasons": {},
    "reason_ids": {},
    "valid": True,
}

MOCK_SERVICE_STATUS_ENTITLEMENT = {
    "status": "Current",
    "status_id": "valid",
    "reasons": {},
    "reason_ids": {},
    "valid": True,
}


class TestStatusCommand(SubManFixture):
    command_class = managercli.StatusCommand

    def setUp(self):
        super(TestStatusCommand, self).setUp()
        self.cc = self.command_class()
        patcher = patch("subscription_manager.cli_command.status.entitlement")
        self.entitlement_mock = patcher.start()
        self.addCleanup(patcher.stop)
        self.mock_entitlement_instance = Mock()
        self.mock_entitlement_instance.get_status = Mock(return_value=MOCK_SERVICE_STATUS_SCA)
        self.entitlement_mock.EntitlementService = Mock(return_value=self.mock_entitlement_instance)

    def test_disabled_status_sca_mode(self):
        """
        Test status, when SCA mode is used
        """
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "disabled"})
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        mock_cert = Mock()
        mock_cert.entitlement_type = CONTENT_ACCESS_CERT_TYPE
        cert_list = [mock_cert]
        self.cc.entitlement_dir = Mock()
        self.cc.entitlement_dir.list_with_content_access = Mock(return_value=cert_list)
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertIn("Overall Status: Disabled", cap.out)
        self.assertIn("Content Access Mode is set to Simple Content Access.", cap.out)
        self.assertIn("This host has access to content, regardless of subscription status.", cap.out)
        self.assertIn("System Purpose Status: Disabled", cap.out)

    def test_current_status_entitlement_mode(self):
        """
        Test status, when old entitlement mode is used
        """
        # Note that server sent response with "Current" status
        self.mock_entitlement_instance.get_status = Mock(return_value=MOCK_SERVICE_STATUS_ENTITLEMENT)
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "valid"})
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        # There is not SCA certificate (only old entitlement)
        mock_cert = Mock()
        mock_cert.entitlement_type = "entitlement"
        cert_list = [mock_cert]
        self.cc.entitlement_dir = Mock()
        self.cc.entitlement_dir.list_with_content_access = Mock(return_value=cert_list)
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertIn("Overall Status: Current", cap.out)
        self.assertIn("System Purpose Status: Matched", cap.out)

    def test_status_content_access_mode_changed(self):
        """
        Test status, when old entitlement mode is used
        """
        # Note that server sent response with "Current" status
        self.mock_entitlement_instance.get_status = Mock(return_value=MOCK_SERVICE_STATUS_ENTITLEMENT)
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "valid"})
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        # But entitlement directory still contain SCA certificate
        mock_cert = Mock()
        mock_cert.entitlement_type = CONTENT_ACCESS_CERT_TYPE
        cert_list = [mock_cert]
        self.cc.entitlement_dir = Mock()
        self.cc.entitlement_dir.list_with_content_access = Mock(return_value=cert_list)
        self.cc.entcertlib = Mock()
        # sub-man should be able to resurrect from this situation
        with Capture() as cap:
            self.cc._do_command()
        self.assertIn("Overall Status: Current", cap.out)
        self.assertIn("System Purpose Status: Matched", cap.out)

    def test_purpose_status_success(self):
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "valid"})
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertTrue("System Purpose Status: Matched" in cap.out)

    def test_purpose_status_consumer_lack(self):
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "unknown"})
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertTrue("System Purpose Status: Unknown" in cap.out)

    def test_purpose_status_consumer_no_capability(self):
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance({"status": "unknown"})
        self.cc.cp._capabilities = []
        self.cc.options = Mock()
        self.cc.options.on_date = None
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertTrue("System Purpose Status: Unknown" in cap.out)

    def test_purpose_status_mismatch(self):
        self.cc.consumerIdentity = StubConsumerIdentity
        self.cc.cp = StubUEP()
        self.cc.cp.setSyspurposeCompliance(
            {"status": "mismatched", "reasons": ["unsatisfied usage: Production"]}
        )
        self.cc.cp._capabilities = ["syspurpose"]
        self.cc.options = Mock()
        self.cc.options.on_date = None
        self.cc.entcertlib = Mock()
        with Capture() as cap:
            self.cc._do_command()
        self.assertTrue("System Purpose Status: Mismatched" in cap.out)
        self.assertTrue("unsatisfied usage: Production" in cap.out)



# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

import unittest
from unittest import mock
from unittest.mock import patch, Mock

import socket
import requests

from rhsmlib.facts import cloud_facts

from cloud_what.providers import aws, azure, gcp

from subscription_manager import injection as inj

AWS_METADATA = """
{
  "accountId" : "012345678900",
  "architecture" : "x86_64",
  "availabilityZone" : "eu-central-1b",
  "billingProducts" : [ "bp-0124abcd", "bp-63a5400a" ],
  "devpayProductCodes" : null,
  "marketplaceProductCodes" : null,
  "imageId" : "ami-0123456789abcdeff",
  "instanceId" : "i-abcdef01234567890",
  "instanceType" : "m5.large",
  "kernelId" : null,
  "pendingTime" : "2020-02-02T02:02:02Z",
  "privateIp" : "12.34.56.78",
  "ramdiskId" : null,
  "region" : "eu-central-1",
  "version" : "2017-09-30"
}
"""

AZURE_METADATA = """
{
    "compute": {
        "azEnvironment": "AzurePublicCloud",
        "customData": "",
        "location": "westeurope",
        "name": "foo-bar",
        "offer": "RHEL",
        "osType": "Linux",
        "placementGroupId": "",
        "plan": {
            "name": "",
            "product": "",
            "publisher": ""
        },
        "platformFaultDomain": "0",
        "platformUpdateDomain": "0",
        "provider": "Microsoft.Compute",
        "publicKeys": [
            {
                "keyData": "ssh-rsa SOMEpublicSSHkey user@localhost.localdomain",
                "path": "/home/user/.ssh/authorized_keys"
            }
        ],
        "publisher": "RedHat",
        "resourceGroupName": "foo-bar",
        "resourceId": "/subscriptions/01234567-0123-0123-0123-012345679abc/resourceGroups/foo-bar/providers/Microsoft.Compute/virtualMachines/foo",
        "sku": "8.1-ci",
        "storageProfile": {
            "dataDisks": [],
            "imageReference": {
                "id": "",
                "offer": "RHEL",
                "publisher": "RedHat",
                "sku": "8.1-ci",
                "version": "latest"
            },
            "osDisk": {
                "caching": "ReadWrite",
                "createOption": "FromImage",
                "diskSizeGB": "64",
                "encryptionSettings": {
                    "enabled": "false"
                },
                "image": {
                    "uri": ""
                },
                "managedDisk": {
                    "id": "/subscriptions/01234567-0123-0123-0123-012345679abc/resourceGroups/FOO-BAR/providers/Microsoft.Compute/disks/foo_OsDisk_1_b21768daf38e48c6a0db7cff1f054b03",
                    "storageAccountType": ""
                },
                "name": "FOO_OsDisk_1_b21768daf38e48c6a0db7cff1f054b03",
                "osType": "Linux",
                "vhd": {
                    "uri": ""
                },
                "writeAcceleratorEnabled": "false"
            }
        },
        "subscriptionId": "01234567-0123-0123-0123-012345679abc",
        "tags": "",
        "version": "8.1.2020042511",
        "vmId": "12345678-1234-1234-1234-123456789abc",
        "vmScaleSetName": "",
        "vmSize": "Standard_D2s_v3",
        "zone": ""
    },
    "network": {
        "interface": [
            {
                "ipv4": {
                    "ipAddress": [
                        {
                            "privateIpAddress": "172.16.2.5",
                            "publicIpAddress": "1.2.3.4"
                        }
                    ],
                    "subnet": [
                        {
                            "address": "172.16.2.0",
                            "prefix": "24"
                        }
                    ]
                },
                "ipv6": {
                    "ipAddress": []
                },
                "macAddress": "000D3A123456"
            }
        ]
    }
}
"""  # noqa: E501

GCP_JWT_TOKEN = """eyJhbGciOiJSUzI1NiIsImtpZCI6IjZhOGJhNTY1MmE3MDQ0MTIxZDRmZWR\
hYzhmMTRkMTRjNTRlNDg5NWIiLCJ0eXAiOiJKV1QifQ.eyJhdWQiOiJodHRwczovL3N1YnNjcmlwdG\
lvbi5yaHNtLnJlZGhhdC5jb206NDQzL3N1YnNjcmlwdGlvbiIsImF6cCI6IjEwNDA3MDk1NTY4MjI5\
ODczNjE0OSIsImVtYWlsIjoiMTYxOTU4NDY1NjEzLWNvbXB1dGVAZGV2ZWxvcGVyLmdzZXJ2aWNlYW\
Njb3VudC5jb20iLCJlbWFpbF92ZXJpZmllZCI6dHJ1ZSwiZXhwIjoxNjE2NTk5ODIzLCJnb29nbGUi\
OnsiY29tcHV0ZV9lbmdpbmUiOnsiaW5zdGFuY2VfY3JlYXRpb25fdGltZXN0YW1wIjoxNjE2NTk1OD\
Q3LCJpbnN0YW5jZV9pZCI6IjI1ODkyMjExNDA2NzY3MTgwMjYiLCJpbnN0YW5jZV9uYW1lIjoiaW5z\
dGFuY2UtMSIsImxpY2Vuc2VfaWQiOlsiNTczMTAzNTA2NzI1NjkyNTI5OCJdLCJwcm9qZWN0X2lkIj\
oiZmFpci1raW5nZG9tLTMwODUxNCIsInByb2plY3RfbnVtYmVyIjoxNjE5NTg0NjU2MTMsInpvbmUi\
OiJ1cy1lYXN0MS1iIn19LCJpYXQiOjE2MTY1OTYyMjMsImlzcyI6Imh0dHBzOi8vYWNjb3VudHMuZ2\
9vZ2xlLmNvbSIsInN1YiI6IjEwNDA3MDk1NTY4MjI5ODczNjE0OSJ9.XQKeqMAvsH2T2wsdN97jlm5\
2DzLfix3DTMCu9QCuhSKLEk1xHYOYtvh5Yzn7j-tbZtV-siyPAfpGZO3Id87573OVGgohN3q7Exlf9\
CEIHa1-X7zLyiyIlyrnfQJ1aGHeH6y7gb_tWxHFLJRzhulfkfJxSDn5fEBSgBqbjzCr9unQgMkuzQ3\
uui2BbIbALmOpY6D-IT71mgMDZ_zm4G6q-Mh0nIMkDWhmQ8pa3RAVqqBMBYJninKLdCD8eQzIlDhtI\
zwmYGLrsJMktFF3pJFCqEFv1rKZy_OUyV4JOkOLtXbKnwxqmFTq-2SP0KtUWjDy1-U8GnVDptISjOf\
2O9FaLA
"""

# The AWS instance ID has to be the same as "instance_id" in AWS_METADATA
AWS_INSTANCE_ID = "i-abcdef01234567890"
AWS_ACCOUNT_ID = "012345678900"
AWS_BILLING_PRODUCTS = "bp-0124abcd bp-63a5400a"
# The Azure instance ID has to be the same as "vmId" in AZURE_METADATA
# values for "sku" an "offer" has to be same as in AZURE_METADATA
AZURE_INSTANCE_ID = "12345678-1234-1234-1234-123456789abc"
AZURE_SKU = "8.1-ci"
AZURE_OFFER = "RHEL"


def mock_prepare_request(request):
    return request


class TestCloudCollector(unittest.TestCase):
    def setUp(self):
        aws.AWSCloudProvider._instance = None
        aws.AWSCloudProvider._initialized = False
        azure.AzureCloudProvider._instance = None
        azure.AzureCloudProvider._initialized = False
        gcp.GCPCloudProvider._instance = None
        gcp.GCPCloudProvider._initialized = False
        super(TestCloudCollector, self).setUp()
        self.mock_facts = mock.Mock()
        inj.provide(inj.FACTS, self.mock_facts)
        # Azure and GCP
        self.requests_patcher = patch("cloud_what._base_provider.requests")
        self.requests_mock = self.requests_patcher.start()
        self.addCleanup(self.requests_patcher.stop)

    @patch("cloud_what.providers.aws.requests.Session", name="test_get_aws_facts.mock_session_class")
    def test_get_aws_facts(self, mock_session_class):
        """
        Test getting AWS facts (instance ID, accountID and billingProducts)
        """
        mock_result = Mock(name="_test_get_aws_facts.mock_result")
        mock_result.status_code = 200
        mock_result.text = AWS_METADATA
        mock_session = Mock(name="_test_get_aws_facts.mock_session")
        mock_session.send = Mock(return_value=mock_result, name="_test_get_aws_facts.mock_session.send")
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "4.2.amazon"}
        )

        facts = self.collector.get_all()

        self.assertIn("aws_instance_id", facts)
        self.assertEqual(facts["aws_instance_id"], AWS_INSTANCE_ID)
        self.assertIn("aws_account_id", facts)
        self.assertEqual(facts["aws_account_id"], AWS_ACCOUNT_ID)
        self.assertIn("aws_billing_products", facts)
        self.assertEqual(facts["aws_billing_products"], AWS_BILLING_PRODUCTS)
        self.assertIn("aws_marketplace_product_codes", facts)
        self.assertEqual(facts["aws_marketplace_product_codes"], None)

    @patch("cloud_what.providers.aws.requests.Session", name="mock_session_class")
    def test_get_aws_facts_with_null_billing_products(self, mock_session_class):
        """
        Billing products could be null in some cases (not RHEL)
        """
        mock_result = Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = """
{
  "accountId" : "012345678900",
  "architecture" : "x86_64",
  "availabilityZone" : "eu-central-1b",
  "billingProducts" : null,
  "devpayProductCodes" : null,
  "marketplaceProductCodes" : null,
  "imageId" : "ami-0123456789abcdeff",
  "instanceId" : "i-abcdef01234567890",
  "instanceType" : "m5.large",
  "kernelId" : null,
  "pendingTime" : "2020-02-02T02:02:02Z",
  "privateIp" : "12.34.56.78",
  "ramdiskId" : null,
  "region" : "eu-central-1",
  "version" : "2017-09-30"
}
        """
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_session.send")
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "4.2.amazon"}
        )
        facts = self.collector.get_all()

        self.assertIn("aws_instance_id", facts)
        self.assertEqual(facts["aws_instance_id"], AWS_INSTANCE_ID)
        self.assertIn("aws_account_id", facts)
        self.assertEqual(facts["aws_account_id"], AWS_ACCOUNT_ID)
        self.assertIn("aws_billing_products", facts)
        self.assertEqual(facts["aws_billing_products"], None)
        self.assertIn("aws_marketplace_product_codes", facts)
        self.assertEqual(facts["aws_marketplace_product_codes"], None)

    def test_get_azure_facts(self):
        """
        Test getting Azure facts instance ID (vmId) from metadata provided by Azure cloud provider
        """
        self.requests_mock.Request = Mock(name="mock_Request")
        mock_result = Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = AZURE_METADATA
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_send")
        mock_session.prepare_request = Mock(name="mock_prepare_request")
        mock_session.hooks = {"response": []}
        self.requests_mock.Session = Mock(return_value=mock_session, name="mock_Session")

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={
                "virt.is_guest": True,
                "dmi.chassis.asset_tag": "7783-7084-3265-9085-8269-3286-77",
            }
        )
        facts = self.collector.get_all()

        # azure_instance_id should be included in the facts
        self.assertIn("azure_instance_id", facts)
        self.assertEqual(facts["azure_instance_id"], AZURE_INSTANCE_ID)
        # some other azure facts should be included in facts too
        self.assertIn("azure_sku", facts)
        self.assertEqual(facts["azure_sku"], AZURE_SKU)
        self.assertIn("azure_offer", facts)
        self.assertEqual(facts["azure_offer"], AZURE_OFFER)

    @patch("cloud_what.providers.gcp.GCPCloudProvider._write_token_to_cache_file")
    @patch("cloud_what.providers.gcp.GCPCloudProvider._get_metadata_from_cache")
    def test_get_gcp_facts(self, mock_get_metadata_from_cache, mock_write_token_to_cache_file):
        """
        Test getting GCP instance ID from metadata provided by GCP cloud provider
        """
        self.requests_mock.Request = Mock(name="mock_Request")
        mock_result = Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = GCP_JWT_TOKEN
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_send")
        mock_session.prepare_request = Mock(name="mock_prepare_request")
        mock_session.hooks = {"response": []}
        self.requests_mock.Session = Mock(return_value=mock_session, name="mock_Session")

        mock_get_metadata_from_cache.return_value = None
        mock_write_token_to_cache_file.return_value = None

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.vendor": "google"}
        )
        facts = self.collector.get_all()

        self.assertIn("gcp_instance_id", facts)
        self.assertEqual(facts["gcp_instance_id"], "2589221140676718026")
        self.assertIn("gcp_license_codes", facts)
        self.assertEqual(facts["gcp_license_codes"], "5731035067256925298")

    @patch("cloud_what.providers.aws.requests.Session", name="mock_session_class")
    def test_get_not_aws_instance(self, mock_session_class):
        """
        Test that AWS instance ID is not included in facts, when VM is not running on the AWS public cloud
        """
        mock_result = mock.Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = '{"foo": "bar"}'
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_session.send")
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session
        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "Foo"}
        )
        facts = self.collector.get_all()

        self.assertNotIn("aws_instance_id", facts)

    @patch("cloud_what.providers.aws.requests.Session", name="mock_session_class")
    def test_get_bad_json(self, mock_session_class):
        """
        Test parsing some string that is not Json document
        """
        mock_result = mock.Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = "not json document"
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_session.send")
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session
        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "4.2.amazon"}
        )
        facts = self.collector.get_all()

        self.assertNotIn("aws_instance_id", facts)

    @patch("cloud_what.providers.aws.requests.Session", name="mock_session_class")
    def test_get_timeout(self, mock_session_class):
        """
        Test ensures that exception is captured and does not impede
        """
        mock_result = mock.Mock(name="mock_result")
        mock_result.status_code = 200
        mock_result.text = AWS_METADATA
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_session.send")
        mock_session.send.side_effect = socket.timeout
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session
        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "4.2.amazon"}
        )
        facts = self.collector.get_all()

        self.assertNotIn("aws_instance_id", facts)

    @patch("cloud_what.providers.aws.requests.Session", name="mock_session_class")
    def test_get_http_error(self, mock_session_class):
        """
        test ensures that exception is captured and does not impede
        """
        mock_result = mock.Mock(name="mock_result")
        mock_result.status_code = 500
        mock_result.text = "error"
        mock_session = Mock(name="mock_session")
        mock_session.send = Mock(return_value=mock_result, name="mock_session.send")
        mock_session.side_effect = requests.exceptions.HTTPError()
        mock_session.prepare_request = mock_prepare_request
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session
        # We need to patch only Session in this case
        self.requests_patcher.stop()

        self.collector = cloud_facts.CloudFactsCollector(
            collected_hw_info={"virt.is_guest": True, "dmi.bios.version": "4.2.amazon"}
        )
        facts = self.collector.get_all()

        self.assertNotIn("aws_instance_id", facts)



# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
from typing import Any, Dict

import dbus
import json
from unittest import mock

from rhsmlib.dbus.objects import AttachDBusObject

from test.rhsmlib.base import DBusServerStubProvider
from test.rhsmlib.services.test_attach import CONTENT_JSON


class TestAttachDBusObject(DBusServerStubProvider):
    dbus_class = AttachDBusObject
    dbus_class_kwargs: Dict[str, Any] = {}

    @classmethod
    def setUpClass(cls) -> None:
        is_simple_content_access_patch = mock.patch(
            "rhsmlib.dbus.objects.attach.is_simple_content_access",
            name="is_simple_content_access",
        )
        cls.patches["is_simple_content_access"] = is_simple_content_access_patch.start()
        cls.addClassCleanup(is_simple_content_access_patch.stop)

        is_registered_patch = mock.patch(
            "rhsmlib.dbus.base_object.BaseObject.is_registered",
            name="is_registered",
        )
        cls.patches["is_registered"] = is_registered_patch.start()
        cls.addClassCleanup(is_registered_patch.stop)

        update_patch = mock.patch(
            "subscription_manager.certlib.BaseActionInvoker.update",
            name="update",
        )
        cls.patches["update"] = update_patch.start()
        cls.addClassCleanup(update_patch.stop)

        super().setUpClass()

    def setUp(self) -> None:
        self.patches["is_simple_content_access"].return_value = False
        self.patches["is_registered"].return_value = True
        self.patches["update"].return_value = None

        AttachService_patch = mock.patch(
            "rhsmlib.dbus.objects.attach.AttachService",
            name="AttachService",
            autospec=True,
        )
        self.mock_attach = AttachService_patch.start().return_value
        self.addCleanup(AttachService_patch.stop)

        super().setUp()

    def test_PoolAttach(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, self.LOCALE)
        self.assertEqual(result, expected)

    def test_PoolAttach__proxy(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(
            self.obj,
            ["x", "y"],
            1,
            {
                "proxy_hostname": "proxy.company.com",
                "proxy_port": "3128",
                "proxy_user": "user",
                "proxy_password": "password",
            },
            self.LOCALE,
        )
        self.assertEqual(result, expected)

    def test_PoolAttach__de(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, "de")
        self.assertEqual(expected, result)

    def test_PoolAttach__de_DE(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, "de_DE")
        self.assertEqual(expected, result)

    def test_PoolAttach__de_DE_utf8(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, "de_DE.utf-8")
        self.assertEqual(expected, result)

    def test_PoolAttach__de_DE_UTF8(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, "de_DE.UTF-8")
        self.assertEqual(expected, result)

    def test_PoolAttach__sca(self):
        self.patches["is_simple_content_access"].return_value = True
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        # TODO: Change to assertRaises when auto-attach is not supported in SCA mode
        # BZ 2049101, BZ 2049620

        expected = [json.dumps(CONTENT_JSON), json.dumps(CONTENT_JSON)]
        result = self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, "de_DE.UTF-8")
        self.assertEqual(expected, result)

    def test_PoolAttach__not_registered(self):
        self.patches["is_registered"].return_value = False
        self.mock_attach.attach_pool.return_value = CONTENT_JSON

        with self.assertRaisesRegex(dbus.DBusException, "requires the consumer to be registered"):
            self.obj.PoolAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, self.LOCALE)

    def test_AutoAttach(self):
        self.mock_attach.attach_auto.return_value = CONTENT_JSON

        expected = json.dumps(CONTENT_JSON)
        result = self.obj.AutoAttach.__wrapped__(self.obj, "service_level", {}, self.LOCALE)
        self.assertEqual(expected, result)

    def test_AutoAttach__sca(self):
        self.patches["is_simple_content_access"].return_value = True
        self.mock_attach.attach_auto.return_value = CONTENT_JSON

        # TODO: Change to assertRaises when auto-attach is not supported in SCA mode
        # BZ 2049101, BZ 2049620

        expected = json.dumps(CONTENT_JSON)
        result = self.obj.AutoAttach.__wrapped__(self.obj, "service_level", {}, self.LOCALE)
        self.assertEqual(expected, result)

    def test_AutoAttach__not_registered(self):
        self.mock_attach.attach_pool.return_value = CONTENT_JSON
        self.patches["is_registered"].return_value = False

        with self.assertRaisesRegex(dbus.DBusException, "requires the consumer to be registered"):
            self.obj.AutoAttach.__wrapped__(self.obj, ["x", "y"], 1, {}, self.LOCALE)



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os

import rhsm.connection as connection
import subscription_manager.injection as inj

from rhsm.connection import UnauthorizedException, ProxyException

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.abstract_syspurpose import AbstractSyspurposeCommand
from subscription_manager.cli_command.cli import (
    ERR_NOT_REGISTERED_CODE,
    ERR_NOT_REGISTERED_MSG,
    handle_exception,
)
from subscription_manager.cli_command.org import OrgCommand
from subscription_manager.i18n import ugettext as _

from syspurpose.files import SyncedStore

log = logging.getLogger(__name__)


class ServiceLevelCommand(AbstractSyspurposeCommand, OrgCommand):
    def __init__(self, subparser=None):

        shortdesc = _("Show or modify the system purpose service-level setting")
        super(ServiceLevelCommand, self).__init__(
            "service-level",
            subparser,
            shortdesc,
            False,
            attr="service_level_agreement",
            commands=["set", "unset", "show", "list"],
        )
        self._add_url_options()

        self.identity = inj.require(inj.IDENTITY)

    def _validate_options(self):

        if self.options.set:
            self.options.set = self.options.set.strip()

        # Assume --show if run with no args:
        if (
            not self.options.list
            and not self.options.show
            and not self.options.set
            and not self.options.set == ""
            and not self.options.unset
        ):
            self.options.show = True

        if self.options.org and not self.options.list and not self.options.set:
            system_exit(os.EX_USAGE, _("Error: --org is only supported with the --list or --set option"))

        if not self.is_registered():
            if self.options.list:
                if not (self.options.username and self.options.password) and not self.options.token:
                    system_exit(
                        os.EX_USAGE,
                        _(
                            "Error: you must register or specify --username "
                            "and --password to list service levels"
                        ),
                    )
            elif self.options.unset or self.options.set:
                pass  # RHBZ 1632248 : User should be able to set/unset while not registered.
            elif self.options.show:
                pass  # When system is not registered, then user should have ability to display current value
            else:
                system_exit(ERR_NOT_REGISTERED_CODE, ERR_NOT_REGISTERED_MSG)

        if self.is_registered() and (
            getattr(self.options, "username", None)
            or getattr(self.options, "password", None)
            or getattr(self.options, "token", None)
            or getattr(self.options, "org", None)
            or getattr(self.options, "server_url", None)
        ):
            system_exit(
                os.EX_USAGE,
                _(
                    "Error: --username, --password, --token, --org and --serverurl "
                    "can be used only on unregistered systems"
                ),
            )

    def _do_command(self):
        self._validate_options()
        try:
            # If we have a username/password, we're going to use that, otherwise
            # we'll use the identity certificate. We already know one or the other
            # exists:
            if self.options.token:
                self.cp = self.cp_provider.get_keycloak_auth_cp(self.options.token)
            elif self.options.username and self.options.password:
                self.cp_provider.set_user_pass(self.username, self.password)
                self.cp = self.cp_provider.get_basic_auth_cp()
            elif not self.is_registered() and self.options.show:
                pass
            else:
                # get an UEP as consumer
                self.cp = self.cp_provider.get_consumer_auth_cp()
        except connection.RestlibException as re:
            log.exception(re)
            log.error("Error: Unable to retrieve service levels: {re}".format(re=re))

            system_exit(os.EX_SOFTWARE, re)
        except Exception as e:
            handle_exception(_("Error: Unable to retrieve service levels."), e)
        else:
            try:
                if self.options.unset:
                    self.unset()
                elif self.options.set is not None:
                    self.set()
                elif self.options.list:
                    self.list_service_levels()
                elif self.options.show:
                    self.show()
                else:
                    self.show()
            except UnauthorizedException as uex:
                handle_exception(_(str(uex)), uex)
            except connection.GoneException as ge:
                raise ge
            except connection.RestlibException as re_err:
                log.exception(re_err)
                log.error("Error: Unable to retrieve service levels: {err}".format(err=re_err))

                system_exit(os.EX_SOFTWARE, re_err)
            except ProxyException as exc:
                system_exit(os.EX_UNAVAILABLE, exc)

    def set(self):
        if self.cp.has_capability("syspurpose"):
            self.store = SyncedStore(uep=self.cp, consumer_uuid=self.identity.uuid)
            super(ServiceLevelCommand, self).set()
        else:
            self.update_service_level(self.options.set)
            print(_('Service level set to: "{val}".').format(val=self.options.set))

    def unset(self):
        if self.cp.has_capability("syspurpose"):
            self.store = SyncedStore(uep=self.cp, consumer_uuid=self.identity.uuid)
            super(ServiceLevelCommand, self).unset()
        else:
            self.update_service_level("")
            print(_("Service level preference has been unset"))

    def update_service_level(self, service_level):
        consumer = self.cp.getConsumer(self.identity.uuid)
        if "serviceLevel" not in consumer:
            system_exit(
                os.EX_UNAVAILABLE,
                _("Error: The service-level command is not supported by the server."),
            )
        self.cp.updateConsumer(self.identity.uuid, service_level=service_level)

    def show(self):
        if self.cp.has_capability("syspurpose"):
            self.store = SyncedStore(uep=self.cp, consumer_uuid=self.identity.uuid)
            super(ServiceLevelCommand, self).show()
        else:
            self.show_service_level()

    def show_service_level(self):
        consumer = self.cp.getConsumer(self.identity.uuid)
        if "serviceLevel" not in consumer:
            system_exit(
                os.EX_UNAVAILABLE,
                _("Error: The service-level command is not supported by the server."),
            )
        service_level = consumer["serviceLevel"] or ""
        if service_level:
            print(_("Current service level: {level}").format(level=service_level))
        else:
            print(_("Service level preference not set"))

    def list_service_levels(self):
        if self.is_registered():
            org_key = self.cp.getOwner(self.identity.uuid)["key"]
        else:
            org_key = self.org

        try:
            slas = self.cp.getServiceLevelList(org_key)
            if len(slas):
                print("+-------------------------------------------+")
                print("           {label}".format(label=_("Available Service Levels")))
                print("+-------------------------------------------+")
                for sla in slas:
                    print(sla)
            else:
                print(
                    _(
                        'There are no available values for the system purpose "{syspurpose_attr}" '
                        "from the available subscriptions in this "
                        "organization."
                    ).format(syspurpose_attr="service_level")
                )
        except UnauthorizedException as e:
            raise e
        except connection.RemoteServerException:
            system_exit(
                os.EX_UNAVAILABLE, _("Error: The service-level command is not supported by the server.")
            )
        except connection.GoneException as ge:
            raise ge
        except connection.RestlibException as e:
            if e.code == 404 and e.msg.find("/servicelevels") > 0:
                system_exit(
                    os.EX_UNAVAILABLE, _("Error: The service-level command is not supported by the server.")
                )
            elif e.code == 404:
                system_exit(os.EX_DATAERR, e)
            else:
                raise e



import sys

from rhsmlib.services import config

from ..test_managercli import TestCliCommand
from subscription_manager import managercli
from subscription_manager.cli_command import config as config_command

from ..fixture import Capture

from unittest.mock import patch


class TestConfigCommand(TestCliCommand):
    command_class = managercli.ConfigCommand

    def setUp(self):
        super(TestConfigCommand, self).setUp()
        self.original_conf = config_command.conf
        config_command.conf = config.Config(self.mock_cfg_parser)

    def tearDown(self):
        super(TestConfigCommand, self).tearDown()
        config_command.conf = self.original_conf

    def test_list(self):
        self.cc.main(["--list"])
        self.cc._validate_options()

    def test_list_default(self):
        hostname = config_command.conf["server"]["hostname"]
        # BZ 1862431
        config_command.conf["rhsmd"] = {}
        config_command.conf["rhsmd"]["processtimeout"] = "300"
        with Capture() as cap:
            self.cc._do_command = self._orig_do_command
            self.cc.main([])
            self.cc._validate_options()
        self.assertTrue(hostname in cap.out)

    def test_remove(self):
        self.cc.main(["--remove", "server.hostname", "--remove", "server.port"])
        self.cc._validate_options()

    def test_config_list(self):
        self.cc._do_command = self._orig_do_command
        self.cc.main(["--list"])

    def test_config(self):
        self.cc._do_command = self._orig_do_command
        # if args is empty we default to sys.argv, so stub it
        with patch.object(sys, "argv", ["subscription-manager", "config"]):
            self.cc.main([])

    def test_set_config(self):
        self.cc._do_command = self._orig_do_command

        baseurl = "https://someserver.example.com/foo"
        self.cc.main(["--rhsm.baseurl", baseurl])
        self.assertEqual(config_command.conf["rhsm"]["baseurl"], baseurl)

    def test_remove_config_default(self):
        with Capture() as cap:
            self.cc._do_command = self._orig_do_command
            self.cc.main(["--remove", "rhsm.baseurl"])
        self.assertTrue("The default value for" in cap.out)

    def test_remove_config_section_does_not_exist(self):
        self.cc._do_command = self._orig_do_command
        self.assertRaises(SystemExit, self.cc.main, ["--remove", "this.doesnotexist"])

    def test_remove_config_key_does_not_exist(self):
        self.cc._do_command = self._orig_do_command
        self.assertRaises(SystemExit, self.cc.main, ["--remove", "rhsm.thisdoesnotexist"])

    def test_remove_config_key_not_dotted(self):
        self.cc._do_command = self._orig_do_command
        self.assertRaises(SystemExit, self.cc.main, ["--remove", "notdotted"])



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os

from rhsmlib.services.refresh import Refresh
import rhsm.connection as connection

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import CliCommand, handle_exception
from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)


class RefreshCommand(CliCommand):
    def __init__(self):
        shortdesc = _("Pull the latest subscription data from the server")
        super(RefreshCommand, self).__init__("refresh", shortdesc, True)
        self.parser.add_argument("--force", action="store_true", help=_("force certificate regeneration"))

    def _do_command(self):
        self.assert_should_be_registered()
        try:
            refresh_service = Refresh(cp=self.cp, ent_cert_lib=self.entcertlib)
            refresh_service.refresh(force=self.options.force)
        except connection.RestlibException as re:
            log.error(re)
            system_exit(os.EX_SOFTWARE, re)
        except Exception as e:
            handle_exception(
                _("Unable to perform refresh due to the following exception: {e}").format(e=e), e
            )
        else:
            print(_("All local data refreshed"))
        self._request_validity_check()



import configparser
from unittest import TestCase
import os
import subprocess
import tempfile

from test import subman_marker_functional, subman_marker_needs_envvars, subman_marker_zypper


@subman_marker_functional
@subman_marker_zypper
@subman_marker_needs_envvars(
    "RHSM_USER", "RHSM_PASSWORD", "RHSM_URL", "RHSM_POOL", "RHSM_TEST_REPO", "RHSM_TEST_PACKAGE"
)
class TestServicePlugin(TestCase):

    SUB_MAN = "PYTHONPATH=./src python -m subscription_manager.scripts.subscription_manager"

    def setUp(self):
        # start in a non-registered state
        subprocess.call("{sub_man} unregister".format(sub_man=self.SUB_MAN), shell=True)

    def has_subman_repos(self):
        repos = configparser.ConfigParser()
        with tempfile.NamedTemporaryFile(suffix=".repo") as repofile:
            subprocess.call("zypper lr -e {0}".format(repofile.name), shell=True)
            repos.read(repofile.name)
        for repo in repos.sections():
            repo_info = dict(repos.items(repo))
            service = repo_info.get("service", None)
            if service == "rhsm":
                return True
        return False

    def test_provides_no_subman_repos_if_unregistered(self):
        self.assertFalse(self.has_subman_repos())

    def test_provides_subman_repos_if_registered_and_subscribed(self):
        subprocess.call(
            "{sub_man} register --username={RHSM_USER} --password={RHSM_PASSWORD} "
            "--serverurl={RHSM_URL}".format(sub_man=self.SUB_MAN, **os.environ),
            shell=True,
        )
        subprocess.call(
            "{sub_man} attach --pool={RHSM_POOL}".format(sub_man=self.SUB_MAN, **os.environ), shell=True
        )
        self.assertTrue(self.has_subman_repos())

    def test_can_download_rpm(self):
        subprocess.check_call(
            "{sub_man} register --username={RHSM_USER} --password={RHSM_PASSWORD} "
            "--serverurl={RHSM_URL}".format(sub_man=self.SUB_MAN, **os.environ),
            shell=True,
        )
        subprocess.check_call(
            "{sub_man} attach --pool={RHSM_POOL}".format(sub_man=self.SUB_MAN, **os.environ), shell=True
        )
        subprocess.check_call(
            "{sub_man} repos --enable={RHSM_TEST_REPO}".format(sub_man=self.SUB_MAN, **os.environ), shell=True
        )

        # remove cached subman packages
        subprocess.call("rm -rf /var/cache/zypp/packages/subscription-manager*", shell=True)
        # remove test package if installed
        subprocess.call(
            "PYTHONPATH=./src zypper --non-interactive rm {RHSM_TEST_PACKAGE}".format(**os.environ),
            shell=True,
        )
        subprocess.call(
            "PYTHONPATH=./src zypper --non-interactive --no-gpg-checks in "
            "--download-only {RHSM_TEST_PACKAGE}".format(**os.environ),
            shell=True,
        )

        subprocess.check_call(
            "test \"$(find /var/cache/zypp/packages/ -name '{RHSM_TEST_PACKAGE}*.rpm' | wc -l)\" "
            "-gt 0".format(**os.environ),
            shell=True,
        )



# Copyright (c) 2010-2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
"""Load and collect DMI data.

"""
import contextlib
import logging
from typing import Dict, List, Union

from rhsmlib.facts import collector
from rhsmlib.facts.dmidecodeparser import DmidecodeParser

log = logging.getLogger(__name__)


class DmidecodeFactCollector(collector.FactsCollector):
    def __init__(
        self,
        prefix: str = None,
        testing: bool = None,
        collected_hw_info: Dict[str, Union[str, int, bool, None]] = None,
    ):
        super(DmidecodeFactCollector, self).__init__(
            prefix=prefix, testing=testing, collected_hw_info=collected_hw_info
        )

        self._dmidecode_output: str = None

    def set_dmidecode_output(self, filename: str):
        self._dmidecode_output = filename

    def get_all(self) -> Dict[str, str]:
        """
        Collect facts from the dmidecode output, if available.

        There are different quirks done to make the facts returned closer
        to the way python-dmidecode used to return them.
        """
        parser = DmidecodeParser()
        try:
            if self._dmidecode_output is not None:
                parser.parse_file(self._dmidecode_output)
            else:
                parser.parse()
        except Exception as exc:
            log.warning("Failed to parse the dmidecode output: {exc}")
            log.exception(exc)
            return {}

        dmiinfo: Dict[str, str] = {}
        # map the various DMI types to the various subtags of "dmi" facts;
        # there can be multiple types for the same subtag, as python-dmidecode
        # aggregated them
        tags = {
            DmidecodeParser.DmiTypes.BIOS_INFORMATION: "dmi.bios.",
            DmidecodeParser.DmiTypes.BIOS_LANGUAGE_INFORMATION: "dmi.bios.",
            DmidecodeParser.DmiTypes.PROCESSOR_INFORMATION: "dmi.processor.",
            DmidecodeParser.DmiTypes.BASEBOARD_INFORMATION: "dmi.baseboard.",
            DmidecodeParser.DmiTypes.SYSTEM_ENCLOSURE_OR_CHASSIS: "dmi.chassis.",
            DmidecodeParser.DmiTypes.SYSTEM_SLOTS: "dmi.slot.",
            DmidecodeParser.DmiTypes.SYSTEM_INFORMATION: "dmi.system.",
            DmidecodeParser.DmiTypes.SYSTEM_CONFIGURATION_OPTIONS: "dmi.system.",
            DmidecodeParser.DmiTypes.MEMORY_DEVICE: "dmi.memory.",
            DmidecodeParser.DmiTypes.PHYSICAL_MEMORY_ARRAY: "dmi.memory.",
            DmidecodeParser.DmiTypes.PORT_CONNECTOR_INFORMATION: "dmi.connector.",
        }
        dmi_type: str
        facts_tag: str
        sections: List[Dict[str, Union[str, List[str]]]]
        for dmi_type, facts_tag in tags.items():
            try:
                sections = parser.get_sections(dmi_type)
            except KeyError:
                continue
            # quirk: use the last handle (likely the one with a higher value)
            # in a similar way to what python-dmidecode did
            section = sections[-1]
            for key, value in section.items():
                if not isinstance(value, str):
                    # we are skipping lists
                    continue

                nkey: str = "".join([facts_tag, key.lower()]).replace(" ", "_")
                nvalue: Union[str, List[str]] = value
                if nvalue.startswith("0x"):
                    # quirk: hex value, lowercase it like python-dmidecode did
                    nvalue = value.lower()
                elif key == "UUID":
                    # quirk: UUID, uppercase it like python-dmidecode did
                    nvalue = value.upper()
                dmiinfo[nkey] = nvalue

        try:
            sections = parser.get_sections(DmidecodeParser.DmiTypes.PROCESSOR_INFORMATION)
        except KeyError:
            pass
        else:
            socket_designations: int = sum(1 for s in sections for k in s.keys() if k == "Socket Designation")
            # Populate how many socket descriptions we have in a faux-fact,
            # so we can use it to munge lscpu info later if needed.
            if socket_designations > 0:
                dmiinfo["dmi.meta.cpu_socket_count"] = str(socket_designations)

        try:
            sections = parser.get_sections(DmidecodeParser.DmiTypes.MEMORY_DEVICE)
        except KeyError:
            pass
        else:
            # quirk: set dmi.memory.size based on the first "useful" value
            # among all the memory devices available
            with contextlib.suppress(StopIteration, KeyError):
                dmiinfo["dmi.memory.size"] = next(
                    s["Size"] for s in sections if s["Size"] != "No Module Installed"
                )

        return dmiinfo



# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
import dbus
from unittest import mock

from rhsmlib.dbus.objects import UnregisterDBusObject

from test.rhsmlib.base import DBusServerStubProvider


class TestUnregisterDBusObject_(DBusServerStubProvider):
    dbus_class = UnregisterDBusObject
    dbus_class_kwargs = {}

    @classmethod
    def setUpClass(cls) -> None:
        is_registered_patch = mock.patch(
            "rhsmlib.dbus.base_object.BaseObject.is_registered",
            name="is_registered",
        )
        cls.patches["is_registered"] = is_registered_patch.start()
        cls.addClassCleanup(is_registered_patch.stop)

        super().setUpClass()

    def test_Unregister__must_be_registered(self):
        self.patches["is_registered"].return_value = False

        with self.assertRaisesRegex(dbus.DBusException, r"requires the consumer to be registered.*"):
            self.obj.Unregister.__wrapped__(self.obj, {}, self.LOCALE)



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import os
import readline

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.user_pass import UserPassCommand
from subscription_manager.i18n import ugettext as _
from subscription_manager.utils import is_interactive


class OrgCommand(UserPassCommand):
    """
    Abstract class for commands that require an org.
    """

    def __init__(self, name, shortdesc=None, primary=False):
        super(OrgCommand, self).__init__(name, shortdesc, primary)
        self._org = None
        if not hasattr(self, "_org_help_text"):
            self._org_help_text = _("specify an organization")
        self.parser.add_argument(
            "--org",
            dest="org",
            metavar="ORG_KEY",
            help=self._org_help_text,
        )

    @staticmethod
    def _get_org(org):
        if not is_interactive():
            system_exit(os.EX_USAGE, _("Error: --org is a required parameter in non-interactive mode."))
        while not org:
            org = input(_("Organization: "))
            readline.clear_history()
        return org

    @property
    def org(self):
        if not self._org:
            if self.options.org is not None:
                self._org = self.options.org
            else:
                owners = self.cp.getOwnerList(self.options.username)
                if len(owners) == 0:
                    system_exit(
                        os.EX_DATAERR,
                        _("Error: User {username} is not member of any organization.").format(
                            username=self.options.username
                        ),
                    )
                elif len(owners) == 1:
                    self._org = owners[0]["key"]
                else:
                    # Get a list of valid owners. Since no owner was specified,
                    # print a hint message showing available owners, before asking
                    # to enter one.
                    org_keys = [owner["key"] for owner in owners]
                    print(
                        _('Hint: User "{name}" is member of following organizations: {orgs}').format(
                            name=self.username, orgs=", ".join(org_keys)
                        )
                    )
                    self._org = self._get_org(self.options.org)
        return self._org
